#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from django.db import models
from django.contrib.auth.models import User
from django.db.models.signals import post_save
from django.dispatch import receiver

import random
import string

class AccountSettings(models.Model):

    class Meta:
        verbose_name_plural = "account settings"


    owner = models.OneToOneField(User)
    daily_summary = models.BooleanField(default=True)
    experiment_mail_notifications_enabled = models.BooleanField(default=True)
    database_notifications_enabled = models.BooleanField(default=True)
    environment_notifications_enabled = models.BooleanField(default=True)


class SupervisionTrack(models.Model):

    #_____ Fields __________

    supervisee           = models.ForeignKey(User, on_delete=models.CASCADE,
        related_name='supervisors')
    godfather            = models.ForeignKey(User, on_delete=models.CASCADE,
        related_name='supervisees')
    is_valid             = models.BooleanField(default=False)
    start_date           = models.DateTimeField(null=True, blank=True)
    expiration_date      = models.DateTimeField(null=True, blank=True)
    last_validation_date = models.DateTimeField(null=True, blank=True)
    supervision_key      = models.CharField(max_length=40, null=True, blank=True)

    def __unicode__(self):
      return u'Godfather: %s, Supervisee, %s, Validity: %s' % (self.godfather.username, self.supervisee.username, self.is_valid)


class Profile(models.Model):
    #_____ Constants __________

    #New account creation 'N'/'W'
    #Godfather acceptance/rejection 'A'/'R'
    #Yearly revalidation/blockage  'Y'/'B'
    NEWUSER           = 'N'
    WAITINGVALIDATION = 'W'
    ACCEPTED          = 'A'
    REJECTED          = 'R'
    YEARREVALIDATION  = 'Y'
    BLOCKED           = 'B'

    PROFILE_STATUS = (
        (NEWUSER, 'New User'),
        (WAITINGVALIDATION, 'Waiting Validation'),
        (ACCEPTED,    'Accepted'),
        (REJECTED,    'Rejected'),
        (YEARREVALIDATION, 'Yearly revalidation'),
        (BLOCKED, 'Blocked no supervisor'),
    )

    #_____ Fields __________

    # This field is required.
    user = models.OneToOneField(User, on_delete=models.CASCADE)

    # Other fields here
    status       = models.CharField(max_length=1, choices=PROFILE_STATUS, default=BLOCKED)
    is_godfather = models.BooleanField(default=False)
    supervision  = models.ManyToManyField(SupervisionTrack, related_name='profiles', blank=True)
    supervision_key = models.CharField(max_length=40, null=True, blank=True)
    registration_date = models.DateTimeField(null=True, blank=True)
    rejection_date = models.DateTimeField(null=True, blank=True)

    def __unicode__(self):
      return u'User: %s' % self.user.username

    def _generate_current_supervision_key(self):
      length = 40
      return ''.join(random.choice(string.ascii_letters + string.digits) for _
          in range(length))

@receiver(post_save, sender=User)
def create_user_profile(sender, instance, created, **kwargs):
    if created:
        Profile.objects.create(user=instance)
    else:
        return

@receiver(post_save, sender=User)
def save_user_profile(sender, instance, **kwargs):
    player, created = Profile.objects.get_or_create(user=instance)
    instance.profile.save()
