#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from django.contrib.auth.models import User, AnonymousUser

from rest_framework import serializers

from .models import Profile, SupervisionTrack
from ..common.models import Contribution
from ..common.fields import JSONSerializerField
from ..ui.templatetags.markup import restructuredtext
from ..common.utils import validate_restructuredtext

import simplejson as json


#----------------------------------------------------------


class UserSerializer(serializers.ModelSerializer):
    username = serializers.SerializerMethodField()
    email = serializers.SerializerMethodField()

    class Meta:
        model = User
        fields = ['username', 'email']

    def get_username(self, obj):
        return obj.username

    def get_email(self, obj):
        return obj.email


#----------------------------------------------------------


class BasicSupervisionTrackSerializer(serializers.ModelSerializer):
    supervisee = UserSerializer()
    godfather = UserSerializer()
    is_valid = serializers.SerializerMethodField()
    start_date = serializers.SerializerMethodField()
    expiration_date = serializers.SerializerMethodField()
    last_validation_date = serializers.SerializerMethodField()
    supervision_key = serializers.SerializerMethodField()

    class Meta:
        model = SupervisionTrack
        fields = ['is_valid']

    #def get_supervisee(self, obj):
    #    return obj.supervisee

    #def get_godfather(self, obj):
    #    return obj.godfather

    def get_is_valid(self, obj):
        return obj.is_valid

    def get_start_date(self, obj):
        return obj.start_date

    def get_expiration_date(self, obj):
        return obj.expiration_date

    def get_last_validation_date(self, obj):
        return obj.last_validation_date

    def get_supervision_key(self, obj):
        return obj.supervision_key


#----------------------------------------------------------


class FullSupervisionTrackSerializer(BasicSupervisionTrackSerializer):

    class Meta(BasicSupervisionTrackSerializer.Meta):
        fields = ['supervisee', 'godfather', 'is_valid', 'start_date', 'expiration_date','last_validation_date', 'supervision_key']


#----------------------------------------------------------


class SupervisionTrackUpdateSerializer(BasicSupervisionTrackSerializer):
    pass


#----------------------------------------------------------
