/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
*/
/**
 * beat.accounts.dialogs.js
 * Implementation of experiment-related dialogs
 */
if (beat === undefined) var beat = {};
if (beat.accounts === undefined) beat.accounts = {};
if (beat.accounts.dialogs === undefined) beat.accounts.dialogs = {};

/**
 * Implements a modal dialog to renew a user account.
 * The user is presented with a small modal window that will allow
 * him to certify the wish to renew his account. It is possible to cancel
 * the action at any moment by pressing "Cancel" or hitting the ESC key.
 *
 * Parameters:
 *
 *   dialog_id: ID of the DOM element to use
 *   api_url (str): The URL towards the API for renewing the account
 *   redirect_url (str): The URL where to redirect the user if the operation is
 *       successful
 */
beat.accounts.dialogs.modal_renew_account = function(dialog_id, api_url, redirect_url) {

  // Create the dialog
  $('#' + dialog_id).dialog({
    autoOpen: false,
    resizable: false,
    width: Math.min($(window).width() * 0.6, 700),
    position: { my: "center", at: "center", of: window },
    modal: true,
    closeOnEscape: true,
    buttons: [
    {
      id: 'button-' + dialog_id + '-renameit',
      text: 'Revalidate',
      click: function() {

        $.ajaxSetup({
          beforeSend: function(xhr, settings) {
            var csrftoken = $.cookie('csrftoken');
            xhr.setRequestHeader('X-CSRFToken', csrftoken);
          }
        });

        $.ajax({
          type: "PUT",
          url: api_url,
          data: JSON.stringify({
            name: name,
          }),
          contentType: "application/json; charset=utf-8",
          dataType: "json",

          success: function(data) {
            $('#' + dialog_id).dialog("close");
            window.location.href = redirect_url;
          },

          error: function(jqXHR, textStatus, errorThrown) {
            if ((jqXHR.status == 400) && (jqXHR.responseText.length > 0))
              alert(jqXHR.responseText);
            else
              alert('Error: ' + errorThrown);

            $(this).dialog("close");
          }
        });
      }
    },
    {
      id: 'button-' + dialog_id + '-cancel',
      text: 'Cancel',
      click: function() {
        $(this).dialog("close");
      }
    }
    ]
  });

  // Display the dialog
  $('#' + dialog_id).dialog("open");
}

/**
 * Implements a modal dialog to change supervisor.
 * The user is presented with a small modal window that will allow
 * him to add the new supervisor username. It is possible to cancel the action at
 * any moment by pressing "Cancel" or hitting the ESC key.
 *
 * Parameters:
 *
 *   dialog_id: ID of the DOM element to use
 *   old_godfather_username: previous supervisor username
 *   api_url (str): The URL towards the API for changing the supervisor
 *   redirect_url (str): The URL where to redirect the user if the operation is
 *       successful
 */
beat.accounts.dialogs.modal_change_supervisor = function(dialog_id, old_godfather_username, api_url, redirect_url) {

  var split_api_url = api_url.split(old_godfather_username);

  // Create the dialog
  $('#' + dialog_id).dialog({
    autoOpen: false,
    resizable: false,
    width: Math.min($(window).width() * 0.6, 700),
    position: { my: "center", at: "center", of: window },
    modal: true,
    closeOnEscape: true,
    buttons: [
    {
      id: 'button-' + dialog_id + '-renameit',
      text: 'Request',
      click: function() {

      var new_godfather_username = $('#' + dialog_id + ' #new_godfather_username')[0].value;

      var name = $('#' + dialog_id + ' input')[0].value.trim();

        if (name.length == 0)
        {
          alert("You can't enter an empty supervisor username!");
          return;
        }

        if (new_godfather_username == old_godfather_username)
        {
          alert("The new supervisor username is the same as the previous one!");
          return;
        }

        $.ajaxSetup({
          beforeSend: function(xhr, settings) {
            var csrftoken = $.cookie('csrftoken');
            xhr.setRequestHeader('X-CSRFToken', csrftoken);
          }
        });

        $.ajax({
          type: "POST",
          url: split_api_url[0] + new_godfather_username + split_api_url[1],
          data: JSON.stringify({
            name: name,
          }),
          contentType: "application/json; charset=utf-8",
          dataType: "json",

          success: function(data) {
            $('#' + dialog_id).dialog("close");
            window.location.href = redirect_url;
          },

          error: function(jqXHR, textStatus, errorThrown) {
            if ((jqXHR.status == 400) && (jqXHR.responseText.length > 0))
              alert(jqXHR.responseText);
            else
              alert('Error: ' + errorThrown);
          }
        });
      }
    },
    {
      id: 'button-' + dialog_id + '-cancel',
      text: 'Cancel',
      click: function() {
        $(this).dialog("close");
      }
    }
    ]
  });

  // Initialise the dialog content
  $('#' + dialog_id + ' input')[0].value = old_godfather_username;

  // Display the dialog
  $('#' + dialog_id).dialog("open");
}

/**
 * Implements a modal dialog to add a supervisor.
 * The user is presented with a small modal window that will allow
 * him to add to set the supervisor username. It is possible to cancel the action at
 * any moment by pressing "Cancel" or hitting the ESC key.
 *
 * Parameters:
 *
 *   dialog_id: ID of the DOM element to use
 *   api_url (str): The URL towards the API for adding a supervisor
 *   redirect_url (str): The URL where to redirect the user if the operation is
 *       successful
 */
beat.accounts.dialogs.modal_add_supervisor = function(dialog_id, api_url, redirect_url) {

  var split_api_url = api_url.split("nogodfather");

  // Create the dialog
  $('#' + dialog_id).dialog({
    autoOpen: false,
    resizable: false,
    width: Math.min($(window).width() * 0.6, 700),
    position: { my: "center", at: "center", of: window },
    modal: true,
    closeOnEscape: true,
    buttons: [
    {
      id: 'button-' + dialog_id + '-renameit',
      text: 'Request',
      click: function() {

      var new_godfather_username = $('#' + dialog_id + ' #new_godfather_username')[0].value;

      var name = $('#' + dialog_id + ' input')[0].value.trim();

        if (name.length == 0)
        {
          alert("You can't enter an empty supervisor username!");
          return;
        }

        $.ajaxSetup({
          beforeSend: function(xhr, settings) {
            var csrftoken = $.cookie('csrftoken');
            xhr.setRequestHeader('X-CSRFToken', csrftoken);
          }
        });

        $.ajax({
          type: "POST",
          url: split_api_url[0] + new_godfather_username + split_api_url[1],
          data: JSON.stringify({
            name: name,
          }),
          contentType: "application/json; charset=utf-8",
          dataType: "json",

          success: function(data) {
            $('#' + dialog_id).dialog("close");
            window.location.href = redirect_url;
          },

          error: function(jqXHR, textStatus, errorThrown) {
            if ((jqXHR.status == 400) && (jqXHR.responseText.length > 0))
              alert(jqXHR.responseText);
            else
              alert('Error: ' + errorThrown);
          }
        });
      }
    },
    {
      id: 'button-' + dialog_id + '-cancel',
      text: 'Cancel',
      click: function() {
        $(this).dialog("close");
      }
    }
    ]
  });

  // Initialise the dialog content
  $('#' + dialog_id + ' input')[0].value = '';

  // Display the dialog
  $('#' + dialog_id).dialog("open");
}

/**
 * Implements a modal dialog for the supervisor to validate a supervisee
 * account. It is possible to cancel the action at any moment by pressing
 * "Cancel" or hitting the ESC key.
 *
 * Parameters:
 *
 *   dialog_id: ID of the DOM element to use
 *   api_url (str): The URL towards the API to validate a supervisee
 *   redirect_url (str): The URL where to redirect the user if the operation is
 *       successful
 */
beat.accounts.dialogs.modal_validate_supervisee = function(dialog_id, api_url, redirect_url) {

  // Create the dialog
  $('#' + dialog_id).dialog({
    autoOpen: false,
    resizable: false,
    width: Math.min($(window).width() * 0.6, 700),
    position: { my: "center", at: "center", of: window },
    modal: true,
    closeOnEscape: true,
    buttons: [
    {
      id: 'button-' + dialog_id + '-renameit',
      text: 'Accept',
      click: function() {

        $.ajaxSetup({
          beforeSend: function(xhr, settings) {
            var csrftoken = $.cookie('csrftoken');
            xhr.setRequestHeader('X-CSRFToken', csrftoken);
          }
        });

        $.ajax({
          type: "PUT",
          url: api_url,
          data: JSON.stringify({
            name: name,
          }),
          contentType: "application/json; charset=utf-8",
          dataType: "json",

          success: function(data) {
            $('#' + dialog_id).dialog("close");
            window.location.href = redirect_url;
          },

          error: function(jqXHR, textStatus, errorThrown) {
            if ((jqXHR.status == 400) && (jqXHR.responseText.length > 0))
              alert(jqXHR.responseText);
            else
              alert('Error: ' + errorThrown);

            $(this).dialog("close");
          }
        });
      }
    },
    {
      id: 'button-' + dialog_id + '-cancel',
      text: 'Cancel',
      click: function() {
        $(this).dialog("close");
      }
    }
    ]
  });

  // Display the dialog
  $('#' + dialog_id).dialog("open");
}

/**
 * Implements a modal dialog for the supervisor to remove a supervisee
 * account. It is possible to cancel the action at any moment by pressing
 * "Cancel" or hitting the ESC key.
 *
 * Parameters:
 *
 *   dialog_id: ID of the DOM element to use
 *   api_url (str): The URL towards the API to remove a supervisee
 *   redirect_url (str): The URL where to redirect the user if the operation is
 *       successful
 */
beat.accounts.dialogs.modal_remove_supervisee = function(dialog_id, api_url, redirect_url) {

  // Create the dialog
  $('#' + dialog_id).dialog({
    autoOpen: false,
    resizable: false,
    width: Math.min($(window).width() * 0.6, 700),
    position: { my: "center", at: "center", of: window },
    modal: true,
    closeOnEscape: true,
    buttons: [
    {
      id: 'button-' + dialog_id + '-renameit',
      text: 'Reject',
      click: function() {

        $.ajaxSetup({
          beforeSend: function(xhr, settings) {
            var csrftoken = $.cookie('csrftoken');
            xhr.setRequestHeader('X-CSRFToken', csrftoken);
          }
        });

        $.ajax({
          type: "PUT",
          url: api_url,
          data: JSON.stringify({
            name: name,
          }),
          contentType: "application/json; charset=utf-8",
          dataType: "json",

          success: function(data) {
            $('#' + dialog_id).dialog("close");
            window.location.href = redirect_url;
          },

          error: function(jqXHR, textStatus, errorThrown) {
            if ((jqXHR.status == 400) && (jqXHR.responseText.length > 0))
              alert(jqXHR.responseText);
            else
              alert('Error: ' + errorThrown);

            $(this).dialog("close");
          }
        });
      }
    },
    {
      id: 'button-' + dialog_id + '-cancel',
      text: 'Cancel',
      click: function() {
        $(this).dialog("close");
      }
    }
    ]
  });

  // Display the dialog
  $('#' + dialog_id).dialog("open");
}
