#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from django.shortcuts import render
from django.contrib.auth.decorators import login_required
from django.contrib.auth.forms import PasswordChangeForm
from django.contrib import messages
from django.db import models
from django.db.models import Q

from rest_framework.authtoken.models import Token

from .forms import AccountSettingsForm
from .models import AccountSettings
from .models import SupervisionTrack, Profile

@login_required
def account_settings(request):

    account_settings = AccountSettings.objects.get(owner=request.user)
    account_settings_form = AccountSettingsForm(instance=account_settings)
    user = request.user
    password_change_form = PasswordChangeForm(user=user)

    if request.method == 'POST':
        if 'password' in request.POST:
            password_change_form = PasswordChangeForm(data=request.POST, user=user)
            if password_change_form.is_valid():
                password_change_form.save()
                messages.add_message(request, messages.SUCCESS, 'Password changed successfully')

        elif 'token' in request.POST:
            user.auth_token.delete()
            Token.objects.create(user=user)
            messages.add_message(request, messages.SUCCESS, 'Token changed successfully')

        else: #'settings'
            account_settings_form = AccountSettingsForm(data=request.POST, instance=account_settings)
            if account_settings_form.is_valid():
              account_settings_form.save()
              messages.add_message(request, messages.SUCCESS, 'Account settings changed successfully')

    else:
        account_settings_form = AccountSettingsForm(instance=account_settings)
        password_change_form = PasswordChangeForm(user=user)

    supervisiontrack = None
    supervisiontracks_valid = None
    supervisiontracks_pending = None
    if user.profile.is_godfather == False:
        supervisee = user
        if supervisee.profile.supervision_key is not None:
            #There's a key check if there's a valid track
            supervisiontrack = SupervisionTrack.objects.get(supervisee=supervisee, supervisee__profile__supervision_key=models.F('supervision_key'))
        else:
            supervisiontrack = None
    else:
        godfather = user
        #Filter and get all supervision tracks valid and
        supervisiontracks_valid = SupervisionTrack.objects.filter(godfather=godfather, is_valid=True)
        supervisiontracks_pending = SupervisionTrack.objects.filter(godfather=godfather, is_valid=False, start_date=None)

    return render(request,
                  'accounts/settings.html',
                  {'account_settings_form': account_settings_form,
                   'password_change_form': password_change_form,
                   'user': user,
                   'supervisiontrack': supervisiontrack,
                   'supervisiontracks_valid': supervisiontracks_valid,
                   'supervisiontracks_pending': supervisiontracks_pending,
                   'token' : user.auth_token})
