#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings
import beat.web.common.models
import beat.web.dataformats.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('team', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='DataFormat',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('sharing', models.CharField(default=b'P', max_length=1, choices=[(b'P', b'Private'), (b'S', b'Shared'), (b'A', b'Public'), (b'U', b'Usable')])),
                ('name', models.CharField(help_text=b'The name for this object (space-like characters will be automatically replaced by dashes)', max_length=200)),
                ('version', models.PositiveIntegerField(default=1, help_text=b'The version of this object (an integer starting from 1)')),
                ('short_description', models.CharField(default=b'', help_text=b'Describe the object succinctly (try to keep it under 80 characters)', max_length=100, blank=True)),
                ('creation_date', models.DateTimeField(auto_now_add=True, verbose_name=b'Creation date')),
                ('hash', models.CharField(help_text=b'Hashed value of the object contents (<a href="https://docs.python.org/2/library/hashlib.html">SHA256, hexadecimal digest</a>). This field is auto-generated and managed by the platform.', max_length=64, editable=False)),
                ('declaration_file', models.FileField(db_column=b'declaration', upload_to=beat.web.common.models.get_contribution_declaration_filename, storage=beat.web.dataformats.models.DataFormatStorage(), max_length=200, blank=True, null=True)),
                ('description_file', models.FileField(db_column=b'description', upload_to=beat.web.common.models.get_contribution_description_filename, storage=beat.web.dataformats.models.DataFormatStorage(), max_length=200, blank=True, null=True)),
                ('author', models.ForeignKey(related_name='dataformats', to=settings.AUTH_USER_MODEL)),
                ('extend', models.ForeignKey(related_name='extensions', blank=True, to='dataformats.DataFormat', null=True)),
                ('fork_of', models.ForeignKey(related_name='forks', blank=True, to='dataformats.DataFormat', null=True)),
                ('previous_version', models.ForeignKey(related_name='next_versions', blank=True, to='dataformats.DataFormat', null=True)),
                ('referenced_formats', models.ManyToManyField(related_name='referencing', to='dataformats.DataFormat', blank=True)),
                ('shared_with', models.ManyToManyField(related_name='shared_dataformats', to=settings.AUTH_USER_MODEL, blank=True)),
                ('shared_with_team', models.ManyToManyField(related_name='shared_dataformats', to='team.Team', blank=True)),
            ],
            options={
                'verbose_name': 'dataformat',
                'verbose_name_plural': 'dataformats',
            },
        ),
    ]
