#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings
import beat.web.experiments.models
import beat.web.common.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('databases', '0001_initial'),
        ('algorithms', '0001_initial'),
        ('toolchains', '0001_initial'),
        ('backend', '0001_initial'),
        ('team', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Block',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200)),
                ('status', models.CharField(default=b'N', max_length=1, choices=[(b'N', b'Not cached'), (b'P', b'Processing'), (b'C', b'Cached'), (b'F', b'Failed')])),
                ('analyzer', models.BooleanField(default=False)),
                ('creation_date', models.DateTimeField(auto_now_add=True, null=True)),
                ('start_date', models.DateTimeField(null=True, blank=True)),
                ('end_date', models.DateTimeField(null=True, blank=True)),
                ('algorithm', models.ForeignKey(related_name='blocks', to='algorithms.Algorithm')),
                ('environment', models.ForeignKey(related_name='blocks', to='backend.Environment', null=True)),
            ],
        ),
        migrations.CreateModel(
            name='CachedFile',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('hash', models.CharField(unique=True, max_length=64)),
                ('linear_execution_time', models.FloatField(default=0.0)),
                ('speed_up_real', models.FloatField(default=0.0)),
                ('speed_up_maximal', models.FloatField(default=0.0)),
                ('queuing_time', models.FloatField(default=0.0)),
                ('stdout', models.TextField(null=True, blank=True)),
                ('stderr', models.TextField(null=True, blank=True)),
                ('error_report', models.TextField(null=True, blank=True)),
                ('cpu_time', models.FloatField(default=0.0)),
                ('max_memory', models.BigIntegerField(default=0)),
                ('data_read_size', models.BigIntegerField(default=0)),
                ('data_read_nb_blocks', models.IntegerField(default=0)),
                ('data_read_time', models.FloatField(default=0.0)),
                ('data_written_size', models.BigIntegerField(default=0)),
                ('data_written_nb_blocks', models.IntegerField(default=0)),
                ('data_written_time', models.FloatField(default=0.0)),
                ('blocks', models.ManyToManyField(related_name='hashes', to='experiments.Block', blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='Experiment',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('sharing', models.CharField(default=b'P', max_length=1, choices=[(b'P', b'Private'), (b'S', b'Shared'), (b'A', b'Public'), (b'U', b'Usable')])),
                ('name', models.CharField(max_length=200)),
                ('short_description', models.CharField(default=b'', help_text=b'Describe the object succinctly (try to keep it under 80 characters)', max_length=100, blank=True)),
                ('status', models.CharField(default=b'P', max_length=1, choices=[(b'P', b'Pending'), (b'S', b'Scheduled'), (b'R', b'Running'), (b'D', b'Done'), (b'F', b'Failed'), (b'C', b'Canceling')])),
                ('creation_date', models.DateTimeField(auto_now_add=True, null=True)),
                ('start_date', models.DateTimeField(null=True, blank=True)),
                ('end_date', models.DateTimeField(null=True, blank=True)),
                ('declaration_file', models.FileField(db_column=b'declaration', upload_to=beat.web.common.models.get_contribution_declaration_filename, storage=beat.web.experiments.models.DeclarationStorage(), max_length=300, blank=True, null=True)),
                ('description_file', models.FileField(db_column=b'description', upload_to=beat.web.common.models.get_contribution_description_filename, storage=beat.web.experiments.models.DeclarationStorage(), max_length=300, blank=True, null=True)),
                ('hash', models.CharField(max_length=64)),
                ('author', models.ForeignKey(related_name='experiments', to=settings.AUTH_USER_MODEL)),
                ('referenced_algorithms', models.ManyToManyField(related_name='experiments', to='algorithms.Algorithm', blank=True)),
                ('referenced_datasets', models.ManyToManyField(related_name='experiments', to='databases.DatabaseSet', blank=True)),
                ('shared_with', models.ManyToManyField(related_name='shared_experiments', to=settings.AUTH_USER_MODEL, blank=True)),
                ('shared_with_team', models.ManyToManyField(related_name='shared_experiments', to='team.Team', blank=True)),
                ('toolchain', models.ForeignKey(related_name='experiments', to='toolchains.Toolchain')),
            ],
            options={
                'ordering': ['-creation_date'],
            },
        ),
        migrations.CreateModel(
            name='Result',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=200)),
                ('type', models.CharField(max_length=200)),
                ('primary', models.BooleanField(default=False)),
                ('data_value', models.TextField(null=True, blank=True)),
                ('block', models.ForeignKey(related_name='results', to='experiments.Block')),
            ],
        ),
        migrations.AddField(
            model_name='block',
            name='experiment',
            field=models.ForeignKey(related_name='blocks', to='experiments.Experiment'),
        ),
        migrations.AlterUniqueTogether(
            name='experiment',
            unique_together=set([('author', 'toolchain', 'name')]),
        ),
    ]
