#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################


from __future__ import unicode_literals

from django.db import migrations, models


def move_result_to_cache(apps, schema_editor):
    '''Moves the result association from the block to the related cache file'''

    Result = apps.get_model("experiments", "Result")

    total = Result.objects.count()
    if total: print('')
    for i, r in enumerate(Result.objects.order_by('-id')):
        print("Resetting result (%d) %d/%d..." % (r.id, i+1, total))
        r.cache = r.block.hashes.first()
        r.save()


class Migration(migrations.Migration):

    dependencies = [
        ('experiments', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='result',
            name='cache',
            field=models.ForeignKey(related_name='results',
              to='experiments.CachedFile', null=True),
        ),
        migrations.RunPython(move_result_to_cache),
    ]
