#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################


from __future__ import unicode_literals

from django.db import migrations


def dedup_resuls(apps, schema_editor):
    '''Deletes duplicated results (older ones)'''

    Result = apps.get_model("experiments", "Result")

    for i, r in enumerate(Result.objects.order_by('-id')):
        older = Result.objects.filter(name=r.name, id__lt=r.id,
            cache=r.block.hashes.first())
        if older:
            print("Cache %s already contains Result `%s' - keeping " \
                "newest (out of %d)..." % (r.block.hashes.first().hash, r.name,
                  older.count()+1))
            older.delete()


class Migration(migrations.Migration):

    dependencies = [
        ('experiments', '0002_scheduler_addons'),
        ('search', '0002_scheduler_addons'),
    ]

    operations = [
        migrations.RunPython(dedup_resuls),
        ]
