#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2017 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings

import os
import glob

import beat.core.hash


def set_status(apps, schema_editor):
    CachedFile = apps.get_model("experiments", "CachedFile")

    total = CachedFile.objects.count()
    for i, c in enumerate(CachedFile.objects.order_by('id')):

        print("Updating cached file %d/%d (%s, id=%d)..." % \
            (i+1, total, c.hash, c.id))

        abs_path = os.path.join(settings.CACHE_ROOT, beat.core.hash.toPath(c.hash, suffix=''))

        data_files = sorted(glob.glob(abs_path + '*.index')) + \
                     sorted(glob.glob(abs_path + '*.data'))

        checksum_files = sorted(glob.glob(abs_path + '*.index.checksum')) + \
                         sorted(glob.glob(abs_path + '*.data.checksum'))

        if len(data_files) == 0:
            continue

        if len(data_files) != len(checksum_files):
            continue

        cached = True
        for data_file, checksum_file in zip(data_files, checksum_files):
            with open(checksum_file, 'rt') as f:
                recorded = f.read().strip()

            actual = beat.core.hash.hashFileContents(data_file)

            if actual != recorded:
                cached = False
                break

        if cached:
            c.status = CachedFile.CACHED
            c.save()



class Migration(migrations.Migration):

    dependencies = [
        ('experiments', '0006_block_order'),
    ]

    operations = [
        migrations.AddField(
            model_name='cachedfile',
            name='status',
            field=models.CharField(choices=[(b'N', b'Not cached'), (b'P', b'Processing'), (b'C', b'Cached')], default=b'N', max_length=1),
        ),
        migrations.RunPython(set_status),
    ]
