#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2017 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from django.db import models
from django.conf import settings

from ...databases.models import DatabaseSetOutput

from .block import Block


#----------------------------------------------------------


class BlockInputManager(models.Manager):

    def get_by_natural_key(self, name, experiment_author, toolchain_author,
                           toolchain_name, toolchain_version, experiment_name,
                           cache_hash, database_hash):
        block = Block.objects.get_by_natural_key(name, experiment_author,
            toolchain_author, toolchain_name, toolchain_version,
            experiment_name)

        if cache_hash:
            return self.get(cache__hash=cache_hash, block=block)
        else:
            return self.get(database__hash=database_hash, block=block)


#----------------------------------------------------------


class BlockInput(models.Model):

    block = models.ForeignKey(Block, related_name='inputs', null=True,
                              on_delete=models.CASCADE)

    # if the input cames from another block, then this one is set
    cache = models.ForeignKey('CachedFile', related_name='inputs', null=True,
                              on_delete=models.CASCADE)

    # if the input cames from a dataset, then this one is set
    database = models.ForeignKey(DatabaseSetOutput, related_name='blocks',
                                 null=True, on_delete=models.CASCADE)

    channel = models.CharField(max_length=200, default='', blank=True,
                               help_text="Synchronization channel within the toolchain")

    objects = BlockInputManager()


    def natural_key(self):
        cache_hash = self.cache and self.cache.hash
        database_hash = self.database and self.database.hash
        return self.block.natural_key() + (cache_hash, database_hash)
