#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################


'''Utilities for experiment management'''


from django.db.models import Count

from .models import CachedFile

import logging
logger = logging.getLogger(__name__)


def list_orphaned_cachedfiles():
    '''Lists orphaned cache files that do not exist in the disk either'''

    q = CachedFile.objects.annotate(Count('blocks')).filter(blocks__count__lt=1)
    return [c for c in q if not c.exists()]


def cleanup_orphaned_cachedfiles():
    '''Cleans-up orphaned cache files that do not exist in the disk either'''

    for c in list_orphaned_cachedfiles():
        logger.info("Removing orphaned CachedFile object `%s'..." % c.hash)
        c.delete()
