#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from django.shortcuts import render_to_response
from django.shortcuts import get_object_or_404
from django.template import RequestContext, Context, loader
from django.contrib import messages
from django.conf import settings

from .middleware import remove_banner_message


def terms_of_service(request):
    '''Returns an HTML page with the Terms of Service to the user'''

    template = loader.get_template('navigation/terms_of_service.rst')
    contents = template.render(Context())

    # if user is logged-in, make sure to udpate its agreement status
    if request.user.is_authenticated():
        request.user.agreement.version = settings.TERMS_OF_SERVICE_VERSION
        request.user.agreement.save()
    else:
        # equivalent behavior for non-logged in users
        request.session['beat_disclaimer_version'] = \
                settings.LEGAL_DISCLAIMER_VERSION
    remove_banner_message(request)

    return render_to_response(
            'navigation/terms_of_service.html',
            dict(contents=contents),
            context_instance=RequestContext(request),
            )


#------------------------------------------------


def legal_disclaimer(request):
    '''Returns an HTML page with the Legal Disclaimeir to the user'''

    template = loader.get_template('navigation/legal_disclaimer.rst')
    contents = template.render(Context())

    # if user is logged-in, make sure to udpate its agreement status
    if request.user.is_authenticated():
        request.user.agreement.version = settings.TERMS_OF_SERVICE_VERSION
        request.user.agreement.save()
    else:
        # equivalent behavior for non-logged in users
        request.session['beat_disclaimer_version'] = \
                settings.LEGAL_DISCLAIMER_VERSION

    remove_banner_message(request)

    return render_to_response(
            'navigation/legal_disclaimer.html',
            dict(contents=contents),
            context_instance=RequestContext(request),
            )



#------------------------------------------------


def contact(request):
    '''Returns an HTML page with the contact information'''

    return render_to_response(
            'navigation/contact.html',
            dict(),
            context_instance=RequestContext(request),
            )
