/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
*/
//Directive that returns an element which adds buttons on click which adds items
var app = angular.module('plotterparameterApp');

//Directive used to plot a graph
app.directive("plotinfo", function(){

    return {
        restrict: 'E',
        scope: true,
        replace: true,
        //templateUrl: "/reports/partials/reportTable/",
        templateUrl: function(scope, elem, attrs)
        {
            var prefix = elem['urlprefix'];
            var the_url = prefix + "/plotters/partials/plotinfo/";
            return the_url;
        },
        link: function(scope, elem, attrs)
        {
            var data_to_plot = scope.plotters.selected.sample_data;
            $('td#td_' + scope.plotterparameter.name).ready(function(){
              if(scope.plotterparameter != "CREATION_MODE")
              {
                 beat.experiments.utils.displayPlot(
                     scope.url_prefix,
                     $('td#td_figure_plot')[0],
                     {
                       output: scope.plotterparameter,
                       plotter: scope.plotters.selected.name,
                       parameter: scope.plotterparameter,
                       merged: true,
                       sample_data: scope.plotters.selected.sample_data,
                     },
                     [scope.plotters.selected], // available plotters
                     true, // replace contents of tag above?
                     function(){$(this).find('i.fa-spin').hide();});
              }
              else
              {
                 beat.experiments.utils.displayPlot(
                     scope.url_prefix,
                     $('td#td_figure_plot')[0],
                     {
                       output: scope.plotterparameter,
                       plotter: scope.plotters.selected.name,
                       merged: true,
                       sample_data: scope.plotters.selected.sample_data,
                     },
                     [scope.plotters.selected], // available plotters
                     true, // replace contents of tag above?
                     function(){$(this).find('i.fa-spin').hide();});
              }
            });

        },
        controller: ['$scope', function ($scope) {
        }],
    };
});

//Directive used to plot a graph with dynamic params
app.directive("plotinfodynamicparams", function(){

    return {
        restrict: 'E',
        scope: true,
        replace: true,
        //templateUrl: "/reports/partials/reportTable/",
        templateUrl: function(scope, elem, attrs)
        {
            var prefix = elem['urlprefix'];
            var the_url = prefix + "/plotters/partials/plotinfo/";
            return the_url;
        },
        link: function(scope, elem, attrs)
        {
            var data_to_plot = scope.plotters.selected.sample_data;
            $('td#td_' + scope.plotterparameter.name).ready(function(){
              beat.experiments.utils.displayPlot(
                  scope.url_prefix,
                  $('td#td_figure_plot')[0],
                  {
                    output: scope.plotterparameter,
                    plotter: scope.plotters.selected.name,
                    parameter: scope.plotterparameter,
                    merged: true,
                    sample_data: scope.plotters.selected.sample_data,
                    dynamic_params: JSON.stringify(scope.plotterparams_update),
                  },
                  [scope.plotters.selected], // available plotters
                  true, // replace contents of tag above?
                  function(){$(this).find('i.fa-spin').hide();});
            });

        },
        controller: ['$scope', function ($scope) {
        }],
    };
});

//Directive used to generate the selector for plotting graph
app.directive("selectplotter", function($compile)
{
    return {
        link:function(scope, element, attrs)
        {
            element.bind("change", function()
            {
                var plotter_graph_element = document.getElementById('space-for-plotterparameter-graph');
                generate_graph(plotter_graph_element);

            });

            function generate_graph(head_id)
            {
                var html_div_code = "<plotinfo  urlprefix=" + scope.url_prefix +"></plotinfo>"
                $(head_id ).empty();
                angular.element(head_id).append($compile(html_div_code)(scope));
            }

        }
    };
});

//Directive used to generate the selected plotting graph
app.directive("selectedplotter", function($compile)
{
    return {
        link:function(scope, element, attrs)
        {
            var plotter_graph_element = document.getElementById('space-for-plotterparameter-graph');
            generate_graph(plotter_graph_element);

            function generate_graph(head_id)
            {
                var html_div_code = "<plotinfo  urlprefix=" + scope.url_prefix +"></plotinfo>"
                $(head_id ).empty();
                angular.element(head_id).append($compile(html_div_code)(scope));
            }

        }
    };
});

//Directive used to handle save plotterparameter click
app.directive("saveplotterparameter", function()
{
    return {
        link:function(scope, element, attrs)
        {
            element.bind("click", function()
            {
                //No plotter is selected
                if(attrs.plotter == "None")
                {
                    if($("#plotter-selection :selected").text().length > 0)
                    {
                        scope.plotterparameterData.plotter = scope.plotters.selected.id;

                        updatePlotterParameter();
                    }
                }
                else
                {
                    //plotter is selected: parameter tuning
                    scope.plotterparameterData.plotter = scope.plotters.selected.id;
                    scope.plotterparameterData.customdata = scope.plotterparams_update;

                    updatePlotterParameter();
                }
            });

            function updatePlotterParameter()
            {
                scope.plotterparameterFactory.updatePlotterParameter(scope.user, scope.plotterparameter_user, scope.plotterparameter_name, scope.plotterparameter_version, scope.plotterparameterData, scope.url_prefix)
                    .success(function (returnedData)
                    {
                        beat.ui.plotterparameter.plotterparameter_saved('plotterparameter_saved', scope);
                    })
                    .error(function (error)
                    {
                        scope.status = 'Unable to update plotterparameter data: ' + error.message;
                    });

            }

        }
    };
});

//Directive used to handle save plotterparameter click
app.directive("createplotterparameter", function()
{
    return {
        link:function(scope, element, attrs)
        {
            element.bind("click", function()
            {
                //No plotter is selected
                if(attrs.plotter == "None" || attrs.plotter=="")
                {
                    if($("#plotter-selection :selected").text().length == 0)
                    {
                        alert("Please select a plotter first")
                    }
                    else
                    {
                        scope.plotterparams_update.plotter = scope.plotters.selected.id;

                        createPlotterParameter();
                    }
                }
                else
                {
                    //plotter is selected: parameter tuning
                    //should actually never fall here
                    createPlotterParameter();
                }
            });

            function createPlotterParameter()
            {
                scope.plotterparameterFactory.createPlotterParameter(scope.user, scope.plotterparams_update, scope.url_prefix)
                    .success(function (returnedData)
                    {

                        beat.ui.plotterparameter.plotterparameter_created('plotterparameter_created', scope);
                    })
                    .error(function (error)
                    {
                        var error_text = "";
                        $.each(error, function( key, value ) {

                                error_text += key + ": " + value;
                        });
                        scope.status = 'Unable to create plotterparameter:\n' + error_text;

                        alert(scope.status);
                    });

            }
        }
    };
});

//Directive used to handle save plotterparameter click
app.directive("createplotterparameternewversion", function()
{
    return {
        link:function(scope, element, attrs)
        {
            element.bind("click", function()
            {
                //No plotter is selected
                if(attrs.plotter == "None" || attrs.plotter=="")
                {
                    //plotter is selected: parameter tuning
                    //should actually never fall here

                    if($("#plotter-selection :selected").text().length == 0)
                    {
                        alert("Please select a plotter first")
                    }
                    else
                    {
                        scope.plotterparams_update.plotter = scope.plotters.selected.id;

                        createPlotterParameter();
                    }
                }
                else
                {
                    scope.plotterparams_newversion.plotter = scope.plotters.selected.id;
                    scope.plotterparams_newversion.name = scope.plotterparameter_name;
                    scope.plotterparams_newversion.version = (parseInt(scope.plotterparameter_version) + 1).toString();
                    scope.plotterparams_newversion.previous_version = parseInt(scope.plotterparameter_version).toString();
                    scope.plotterparams_newversion.data = scope.plotterparams_update;

                    createPlotterParameter();
                }
            });

            function createPlotterParameter()
            {
                scope.plotterparameterFactory.createPlotterParameter(scope.user, scope.plotterparams_newversion, scope.url_prefix)
                    .success(function (returnedData)
                    {

                        beat.ui.plotterparameter.plotterparameter_created('plotterparameter_created', scope);
                    })
                    .error(function (error)
                    {
                        var error_text = "";
                        $.each(error, function( key, value ) {

                                error_text += key + ": " + value;
                        });
                        scope.status = 'Unable to create plotterparameter:\n' + error_text;

                        alert(scope.status);
                    });

            }
        }
    };
});

//Directive used to handle save plotterparameter click
app.directive("createplotterparameterfork", function()
{
    return {
        link:function(scope, element, attrs)
        {
            element.bind("click", function()
            {
                //No plotter is selected
                if(attrs.plotter == "None" || attrs.plotter=="")
                {
                    //plotter is selected: parameter tuning
                    //should actually never fall here

                    if($("#plotter-selection :selected").text().length == 0)
                    {
                        alert("Please select a plotter first")
                    }
                    else
                    {
                        scope.plotterparams_update.plotter = scope.plotters.selected.id;

                        createPlotterParameter();
                    }
                }
                else
                {
                    scope.plotterparams_fork.plotter = scope.plotters.selected.id;
                    scope.plotterparams_fork.name = scope.plotterparams_update.name;
                    scope.plotterparams_newversion.version = "1";
                    scope.plotterparams_fork.fork_of = {};
                    scope.plotterparams_fork.fork_of['username'] = scope.plotterparameter_user;
                    scope.plotterparams_fork.fork_of['name'] = scope.plotterparameter_name;
                    scope.plotterparams_fork.fork_of['version'] = scope.plotterparameter_version;
                    scope.plotterparams_fork.data = scope.plotterparams_update;

                    scope.plotterparameter_forking = true;

                    createPlotterParameter();
                }
            });

            function createPlotterParameter()
            {
                scope.plotterparameterFactory.createPlotterParameter(scope.user, scope.plotterparams_fork, scope.url_prefix)
                    .success(function (returnedData)
                    {

                        beat.ui.plotterparameter.plotterparameter_created('plotterparameter_created', scope);
                    })
                    .error(function (error)
                    {
                        var error_text = "";
                        $.each(error, function( key, value ) {

                                error_text += key + ": " + value;
                        });
                        scope.status = 'Unable to create plotterparameter:\n' + error_text;

                        alert(scope.status);
                    });

            }
        }
    };
});


//Directive used to append parameters
app.directive("plotparams", function($compile){

    return {
        restrict: 'E',
        scope: true,
        replace: true,
        //templateUrl: "/reports/partials/reportTable/",
        templateUrl: function(scope, elem, attrs)
        {
            var prefix = elem['urlprefix'];
            var the_url = prefix + "/plotters/partials/plotparamsinfo/";
            return the_url;
        },
        link: function(scope, elem, attrs)
        {
            $('#table_plotparams tr:last').ready(function(){
                var params_from_plotter = scope.$parent.plotters.selected.declaration.parameters;

                //Iterate through each plotter parameters and add them
                $.each(params_from_plotter, function( key, value ) {

                    if(key != "content_type" )
                    {
                        //index in table
                        var tag_id = (scope.textdata).indexOf(key);
                        var html_div_code = '';

                        var current_value = value.default;
                        if(key in scope.plotterparams_update)
                        {
                            current_value = scope.plotterparams_update[key];
                        }

                        if(value.type == "string")
                        {
                            html_div_code = '<tr><td>'+ key +'</td><td><input id="tag_input_'+ key +'" class="form-control input-sm" type="'+value.type+'" value="' + current_value + '"  key="' + key + '" textinput></td><td>'+value.type+'</td><td>'+value.description+'</td></tr>';
                        }
                        else if(value.type == "float64")
                        {
                            var minus_button_plus = '<div style="width:15em" class="input-group"><span class="input-group-btn" id="tag_button_' + key + '" defaultvalue="' + current_value + '" type="'+ value.type +'" key="' + key + '" action="minus" buttonplusminus><button type="button" class="btn btn-danger btn-number" data-type="minus" data-field="quant[2]"><span class="glyphicon glyphicon-minus"></span></button></span><input style="text-align:center" id="tag_input_'+ key +'" class="form-control input-number" type="'+value.type+'" value="' + current_value + '"  key="' + key + '" textinput><span class="input-group-btn" id="tag_button_' + key + '" defaultvalue="' + current_value + '" type="'+ value.type +'" key="' + key + '" action="plus" buttonplusminus><button type="button" class="btn btn-success btn-number" data-type="plus" data-field="quant[2]"><span class="glyphicon glyphicon-plus"></span></button></span></div>';
                            html_div_code = '<tr><td>'+ key +'</td><td>'+ minus_button_plus +'</td><td>'+value.type+'</td><td>'+value.description+'</td></tr>';
                        }
                        else if(value.type == "uint16")
                        {
                            var minus_button_plus = '<div style="width:15em" class="input-group"><span class="input-group-btn" id="tag_button_' + key + '" defaultvalue="' + current_value + '" type="'+ value.type +'" key="' + key + '" action="minus" buttonplusminus><button type="button" class="btn btn-danger btn-number" data-type="minus" data-field="quant[2]"><span class="glyphicon glyphicon-minus"></span></button></span><input style="text-align:center" id="tag_input_'+ key +'" class="form-control input-number" type="'+value.type+'" value="' + current_value + '"  key="' + key + '" textinput><span class="input-group-btn" id="tag_button_' + key + '" defaultvalue="' + current_value + '" type="'+ value.type +'" key="' + key + '" action="plus" buttonplusminus><button type="button" class="btn btn-success btn-number" data-type="plus" data-field="quant[2]"><span class="glyphicon glyphicon-plus"></span></button></span></div>';
                            html_div_code = '<tr><td>'+ key +'</td><td>'+ minus_button_plus +'</td><td>'+value.type+'</td><td>'+value.description+'</td></tr>';
                        }
                        else if(value.type == "bool")
                        {
                            var bool_param = '';
                            if(current_value)
                            {
                                bool_param ='<fieldset class="toggle" id="tag_bool_'+ key +'" type="'+value.type+'" value="' + current_value + '"  key="' + key + '" toggleparam><input id="data-policy_'+key+'" type="checkbox" checked="checked"><label for="data-policy"><div class="toggle-button"><div class="toggle-tab"></div></div></label></fieldset>';
                            }
                            else
                            {
                                bool_param ='<fieldset class="toggle" id="tag_bool_'+ key +'" type="'+value.type+'" value="' + current_value + '"  key="' + key + '" toggleparam><input id="data-policy_'+key+'" type="checkbox"><label for="data-policy"><div class="toggle-button"><div class="toggle-tab"></div></div></label></fieldset>';
                            }
                            html_div_code = '<tr><td>'+ key +'</td><td>'+ bool_param +'</td><td>'+value.type+'</td><td>'+value.description+'</td></tr>';

                        }
                        else
                        {
                            $('#table_plotparams > tbody:last-child').append('<tr><td>'+ key +'</td><td>'+value.default+'</td><td>'+value.type+'</td><td>'+value.description+'</td></tr>');
                        }

                        angular.element('#table_plotparams > tbody:last-child').append($compile(html_div_code)(scope));

                    }
                });




            });

            //var data_to_plot = scope.plotters.selected.sample_data;
            //$('td#td_' + scope.plotterparameter.name).ready(function(){
            //  beat.experiments.utils.displayPlot(
            //      scope.url_prefix,
            //      $('td#td_figure_plot')[0],
            //      {
            //        output: scope.plotterparameter,
            //        plotter: scope.plotters.selected.name,
            //        parameter: scope.plotterparameter,
            //        merged: true,
            //        sample_data: scope.plotters.selected.sample_data,
            //      },
            //      [scope.plotters.selected], // available plotters
            //      true, // replace contents of tag above?
            //      function(){$(this).find('i.fa-spin').hide();});
            //});

        },
        controller: ['$scope', function ($scope) {
        }],
    };
});

//Directive used to handle textinput
app.directive("textinput", function()
{
    return {
        link:function(scope, element, attrs)
        {
            // on blur, update the value in scope
            element.bind('propertychange keyup paste', function (blurEvent) {

                var isValid = false;

                if (element.data('old-value') != element.val()) {

                    var updateValue = element.val();
                    //string check
                    if(attrs.type == "string")
                    {
                        isValid = true;
                        updateValue = element.val();
                    }

                    //integer check
                    else if(attrs.type == "uint16")
                    {
                        //convert value to integer
                        updateValue = parseInt(element.val());
                        if(isNaN(updateValue))
                        {
                            isValid = false;
                            alert("Entered value is not a integer");
                        }
                        else
                        {
                            isValid = true;
                            updateValue = parseInt(element.val());
                        }

                    }

                    //float check
                    else if(attrs.type == "float64")
                    {
                        //convert value to float
                        updateValue = parseFloat(element.val());
                        if(isNaN(updateValue))
                        {
                            isValid = false;
                            alert("Entered value is not a float");
                        }
                        else
                        {
                            isValid = true;
                            updateValue = parseFloat(element.val());
                        }
                    }
                    //check if value is float
                    //updated models
                    if(isValid)
                    {
                        scope.plotterparams_update[attrs.key] = updateValue;
                    }

                    //scope.$apply(function () {
                    //    scope.textdirective = element.val();
                    //    element.data('old-value', element.val());
                    //});
                }
            });

        }
    };
});

//Directive used to handle bool toggle
app.directive("toggleparam", function()
{
    return {
        link:function(scope, element, attrs)
        {
            element.bind("click", function()
            {
                if($("#data-policy_"+attrs.key).prop("checked") == true)
                {
                    $("#data-policy_"+attrs.key).prop("checked", false);
                }
                else
                {
                    $("#data-policy_"+attrs.key).prop("checked", true);
                }

                scope.plotterparams_update[attrs.key] = $("#data-policy_"+attrs.key).prop("checked");
            });
        }
    };
});

//Directive used to handle button plus/minus
app.directive("buttonplusminus", function()
{
    return {
        link:function(scope, element, attrs)
        {
            element.bind("click", function()
            {
                var tomodify = $("#tag_input_"+attrs.key).val();

                if(attrs.type == "uint16")
                {
                    tomodify = parseInt(tomodify);
                    if(attrs.action == "plus")
                    {
                        tomodify = tomodify +1;
                    }
                    else if(attrs.action == "minus")
                    {
                        tomodify = tomodify -1;
                    }
                }
                else if(attrs.type == "float64")
                {
                    tomodify = parseFloat(tomodify);
                    if(attrs.action == "plus")
                    {
                        tomodify = tomodify +0.1;
                    }
                    else if(attrs.action == "minus")
                    {
                        tomodify = tomodify -0.1;
                    }
                    tomodify = tomodify.toFixed(1);
                }
                $("#tag_input_"+attrs.key).val(tomodify);
                scope.plotterparams_update[attrs.key] = tomodify;
            });
        }
    };
});

//Directive used to handle save plotterparameter click
app.directive("plotterparameteritems", function($compile)
{
    return function(scope, element, attrs)
    {
        scope.$on("addParametersElement", function(event)
        {
            //add the plot
            var plotter_graph_element = document.getElementById('space-for-plotterparameter-plot');
            generate_graph(plotter_graph_element);

            //add the parameters
            var parameter_elements = document.getElementById('space-for-plotterparameter-items');
            append_plotterparameters(parameter_elements)

            //functions to operate on graph and parameters

            function generate_graph(head_id)
            {
                var html_div_code = "<plotinfo  urlprefix=" + scope.url_prefix +"></plotinfo>"
                $(head_id ).empty();
                angular.element(head_id).append($compile(html_div_code)(scope));
            }

            function append_plotterparameters(head_id)
            {
                var html_div_code = "<plotparams  urlprefix=" + scope.url_prefix +"></plotparams>"
                $(head_id ).empty();
                angular.element(head_id).append($compile(html_div_code)(scope));
            }

        });
    };
});

//Directive used to handle dynamic testing on graph display
app.directive("testplotterparameters", function($compile)
{
    return function(scope, element, attrs)
    {
        element.bind("click", function()
        {
            //update the plot
            var plotter_graph_element = document.getElementById('space-for-plotterparameter-plot');
            generate_graph(plotter_graph_element);

            //functions to operate on graph and parameters
            function generate_graph(head_id)
            {
                var html_div_code = "<plotinfodynamicparams urlprefix=" + scope.url_prefix +"></plotinfodynamicparams>"
                $(head_id ).empty();
                angular.element(head_id).append($compile(html_div_code)(scope));
            }
        });
    };
});
