/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
*/

/**
 * beat.plotterparameters.dialogs.js
 * Implementation of experiment-related dialogs
 */

/**
 * Setup an input box so that it does automatic experiment name completion, so
 * the user knows what **not** to choose.
 *
 * This function requires 'bootstrap3-typeahead' to be installed
 *
 * Parameters:
 *
 *   selector (str): A jQuery selector for the input box to rig
 *   data (Array): A JS array containing the existing names for the user
 *     experiments
 *   own (str): My own name, if one was set, so we don't mistakenly highlight
 *     my own (previously set) experiment, which continues to be a valid
 *     choice.
 */

if (beat === undefined) var beat = {};
if (beat.plotterparameters === undefined) beat.plotterparameters= {};
if (beat.plotterparameters.dialogs === undefined) beat.plotterparameters.dialogs = {};

beat.plotterparameters.dialogs.name_typeahead = function(selector, data, own) {

  var element = $(selector);

  var check_validity = function() {
    var currval = element.val();
    if (!currval.length) {
      element.parent().removeClass('has-success').addClass('has-error');
      element.siblings('span.glyphicon').removeClass('glyphicon-ok').addClass('glyphicon-remove');
      return; //ok, we don't need to check the rest
    }
    if (own !== undefined && own == currval) {
      element.parent().removeClass('has-error').addClass('has-success');
      element.siblings('span.glyphicon').removeClass('glyphicon-remove').addClass('glyphicon-ok');
      return; //ok, we don't need to check the rest
    }
    if ($.inArray(currval, data) >= 0) {
      element.parent().removeClass('has-success').addClass('has-error');
      element.siblings('span.glyphicon').removeClass('glyphicon-ok').addClass('glyphicon-remove');
    }
    else {
      element.parent().removeClass('has-error').addClass('has-success');
      element.siblings('span.glyphicon').removeClass('glyphicon-remove').addClass('glyphicon-ok');
    }
  }

  check_validity(); //checks if the currently value set is OK
  element.on('keyup', check_validity);

  element.typeahead({
    source: data,
    items: 10, //how many to show at a time
    minLength: 0, //show suggestions even if no text available
    autoSelect: false, //user may want to leave it where it is
    afterSelect: check_validity, //updates after the user selected an item
    showHintOnFocus: true, //dropdown as soon as focus on
  });

}
