#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from django.conf.urls import include, url
from . import views

partial_patterns = [

    url(r'^plotinfo/$',
        views.PartialGroupView.as_view(
            template_name='plotterparameters/partials/plotgraphicinfo.html',
            ),
        name='plotinfo',
        ),

    url(r'^plotparamsinfo/$',
        views.PartialGroupView.as_view(
            template_name='plotterparameters/partials/plotparamsinfo.html',
            ),
        name='plotinfo',
        ),

    url(r'^(?P<template_name>[a-zA-Z_]+\.html)$',
        views.PartialGroupView.as_view(),
        ),

]

urlpatterns = [

    url(r'^partials/',
        include(partial_patterns, namespace='partials'),
        ),

    url(
        r'^$',
        views.list_plotters,
        name='list',
    ),

    url(
        r'^plot/$',
        views.plot,
        name='plot',
    ),

    url(
        r'^plot_sample/$',
        views.plot_sample,
        name='plot_sample',
    ),

    url(
        r'^plot_sample_with_params/$',
        views.plot_sample_with_params,
        name='plot_sample_with_params',
    ),


    url(
        r'^plotterparameter/(?P<author_name>\w+)/new_plotterparameter/$',
        views.create_plotterparameter,
        name='new_plotterparameter',
    ),

    url(
        r'^plotterparameter/(?P<author_name>\w+)/(?P<plotterparameter_name>[-\w]+)/(?P<version>\d+)/new/$',
        views.create_new_version,
        name='new-version',
    ),

    url(
        r'^plotterparameter/(?P<author_name>\w+)/(?P<plotterparameter_name>[-\w]+)/(?P<version>\d+)/fork/$',
        views.fork,
        name='fork',
    ),

    url(
        r'^plotterparameter/diff/(?P<author1>\w+)/(?P<name1>[-\w]+)/(?P<version1>\d+)/(?P<author2>\w+)/(?P<name2>[-\w]+)/(?P<version2>\d+)/$',
        views.diff,
        name='diff',
    ),

    url(
        r'^(?P<author_name>\w+)/plotterparameter/$',
        views.list_plotterparameters,
        name='plotterparameter-list',
    ),

    url(
        r'^plotterparameter/$',
        views.list_plotterparameters_public,
        name='plotterparameter-public-list',
    ),

    url(
        r'^plotterparameter/(?P<author_name>\w+)/(?P<plotterparameter_name>[-\w]+)/(?P<version>\d+)/$',
        views.plotterparameter_for_author,
        name='plotterparameter-author-view',
        ),

    url(
        r'^(?P<author>\w+)/(?P<name>[-\w]+)/(?P<version>\d+)/$',
        views.view,
        name='view',
    ),

    url(
        r'^plotterparameter/(?P<author>\w+)/(?P<name>[-\w]+)/$',
        views.plotterparameter_latest,
        name='plotterparameter-view-latest',
    ),


    url(
        r'^(?P<author>\w+)/(?P<name>[-\w]+)/$',
        views.view,
        name='plotter-view-latest',
    ),

]
