# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2017-04-10 11:21
from __future__ import unicode_literals

from django.db import migrations
import json
import re

# parses a table from the old representation (a list of column objects)
# and returns a table in the new representation
def parse_table(table, precision):
    conv_table = {
            'itemName': 'Table',
            'fields': ['Experiment'],
            'precision': precision
            }

    for row in table:
        if not row['selected']:
            continue

        name = row['name']
        name = re.sub(r'\[.*\]$', '', name)

        if name == 'experiment':
            continue

        if name == 'experiment.execution_time' or name == 'execution_time':
            name = 'total_execution_time'
        elif re.match(r'^execution_time\.', name):
            name = re.sub(r'execution_time', 'linear_execution_time', name)

        if name.startswith('experiment.'):
            name = re.sub(r'experiment\.', '', name)

        if '.' in name:
            segs = name.split('.')
            name = segs[1] + '.' + segs[0]
        conv_table['fields'].append(name)

    return conv_table

# parses a plot from the old representation
# and returns a plot in the new representation
def parse_plot(plot):
    conv_plot = {
        'itemName': plot['required_plotter'][0],
        'name': plot['data']['output'][0],
        'type': plot['required_plotter'][0]
            }
    return conv_plot

# helper func to build the experiment's full name
def experiment_fullname(exp):
    return '%s/%s/%s/%s/%s' % (exp.author.username, exp.toolchain.author.username, exp.toolchain.name, exp.toolchain.version, exp.name)

# helper func to build the analyzer's full name
def analyzer_fullname(report):
    return '%s/%s/%s' % (report.analyzer.author.username, report.analyzer.name, report.analyzer.version)

# converts an old report into the new report format
def move_content_to_groups_format(apps, schema_editor):
    Report = apps.get_model('reports', 'Report')

    for report in Report.objects.all():
        # all of the changes are in the report's content field
        report_content = json.loads(report.content)

        # convert to groups format, but don't touch any report thats
        # already using the new system
        if 'groups' not in report_content:
            # format:
            # ----
            # groups: {
            # 	group1 : {
            # 		experiments: [],
            # 		reportItems: [],
            # 		analyzer: '',
            # 		aliases: {},
            # 		idx: 1
            # 		},
            # 	...
            # }
            # ----
            exps = report.experiments.all()
            obj = {}
            groups = {}
            # default to just one group that contains all experiments/items
            group1 = {
                    # list of experiments in the group
                    'experiments': [ experiment_fullname(e) for e in exps ],
                    'reportItems': [],
                    # analyzer of the report
                    'analyzer': analyzer_fullname(report) if report.analyzer else '',
                    'aliases': {},
                    'idx': 1
                    }

            old_aliases = report_content['alias_experiments'] if 'alias_experiments' in report_content else {}
            # assign aliases
            get_alias = lambda exp_name: old_aliases[exp_name] if exp_name in old_aliases else None
            for e in exps:
                fullname = experiment_fullname(e)
                group1['aliases'][fullname] = get_alias(fullname) or e.name

            count_tables = 0
            count_plots = 0
            for item_name in report_content:
                if item_name == 'floating_point_precision' or item_name == 'alias_experiments':
                    continue

                item = report_content[item_name]
                item_type = 'table' if item_name.startswith('table') else 'plot'
                fpp = report_content['floating_point_precision'] if 'floating_point_precision' in report_content else 10

                converted_content = parse_table(item, fpp) if item_type == 'table' else parse_plot(item)

                converted_id = ''
                if item_type == 'table':
                    converted_id = 'table_' + str(count_tables)
                    count_tables += 1
                else:
                    converted_id = 'plot_' + str(count_plots)
                    count_plots += 1

                converted_item = {
                        'id': converted_id,
                        'content': converted_content
                        }

                group1['reportItems'].append(converted_item)

            groups['group1'] = group1
            obj['groups'] = groups
            report.content = json.dumps(obj)
            report.save()


class Migration(migrations.Migration):

    dependencies = [
        ('reports', '0003_report_last_edited_date'),
    ]

    operations = [
            migrations.RunPython(move_content_to_groups_format)
    ]
