/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
 */

/*
 * downloadLink
 * Desc:
 *  A button to download the content of the report item in the
 *  chosen file format (PNG, JPEG, PDF)
 */
angular.module('reportApp')
.directive("downloadLink", ['PlotService', function(PlotService){
    return {
        scope: {
            domId: '@',
            group: '=',
            itemId: '='
        },
        link: function(scope){
            scope.filetypes = [
                'PNG',
                'JPEG',
                'PDF'
            ];

            // download the img via the invisible link element
            scope.downloadImgs = (e, ftype) => {
                // get plot data URL
                PlotService.downloadPlot(scope.group, scope.itemId, ftype)
                .then(data => {
                    e.preventDefault();
                    // invisible el, see end of template
                    const a = document.querySelector(`#${scope.domId}-download`);

                    a.href = data;
                    a.download = `${scope.domId}-${scope.itemId}.${ftype.toLowerCase()}`;
                    a.click();
                    a.href = '';
                    a.download = '';
                })
                ;
            };
        },
        template: `
<div class='btn-group'>
    <button type='button' class='btn btn-primary dropdown-toggle' data-toggle='dropdown' aria-haspopup='true' aria-expanded='false'>
        Download
        <span class="caret"></span>
    </button>
    <ul class="dropdown-menu">
        <li><a ng-repeat='t in filetypes' ng-click='downloadImgs($event, t)'>{{ t }}</a></li>
    </ul>
</div>
<a id='{{ domId }}-download'></a>
`
    };
}]);
