/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
 */

/*
 * groupAddGroupMenu
 * Desc:
 *  the menu & validating for creating a new group
 */
angular.module('reportApp')
.directive("groupAddGroupMenu", ['GroupsService', function(GroupsService){
    return {
        scope: {
        },
        link: function(scope){
            scope.newGroupName = { val: '' };
            // validates the user input
            scope.hasError = (val) => {
                // if val is undefined, empty, or a dup, its an err
                const isErr = !val || val.length === 0 || GroupsService.groups.find(g => g.name === val);
                // cast to boolean
                return !!isErr;
            };

            // creates a new group if the new group name is valid
            // wipes the input on successful creation
            scope.createGroup = () => {
                if(scope.hasError(scope.newGroupName.val)){
                    return;
                }

                GroupsService.createGroup(scope.newGroupName.val);
                scope.newGroupName.val = '';
            };
        },
        template: `
<form ng-submit='createGroup()'>
    <div class='form-group' ng-class="{'has-error': hasError(newGroupName.val)}">
        <div class="input-group">
            <span class="input-group-btn">
                <button ng-click='createGroup()' class="btn btn-default" type="button">
                    <i class="fa fa-plus" aria-hidden="true"></i>
                </button>
            </span>
            <input required id='createNewGroupInput' ng-model='newGroupName.val' type="text" class="form-control" placeholder="New group name...">
        </div>
    </div>
</form>
`
    };
}]);

