/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
 */

/*
 * groupAddItemsMenu
 * Desc:
 *  the button group for adding report items (plot, table, text) to a group
 */
angular.module('reportApp')
.directive("groupAddItemsMenu", ['ExperimentsService', 'GroupsService', 'PlotService', function(ExperimentsService, GroupsService, PlotService){
    return {
        scope: {
            group: '='
        },
        link: function(scope){
            // finds the id for the next report item of
            // the given type
            // by looking at the existing items
            const getNextItemId = (type) => {
                const formatId = (type, count) => `${type}_${count}`;
                let currCount = 0;
                let nextId = formatId(type, currCount);
                while(scope.group.reportItems.find(i => i.id === nextId)){
                    currCount++;
                    nextId = formatId(type, currCount);
                }

                return nextId;
            };

            scope.plottables = ExperimentsService.plottables;

            // helper func for adding a table
            scope.addNewTable = () => {
                const id = getNextItemId('table');

                // default fields are fields that are meant for tables
                // that have the 'primary' property on the field value set to true
                const defaultFieldsSet = Object.entries(ExperimentsService.tableables)
                // only exps in group
                .filter(([expName, o]) => scope.group.experiments.includes(expName))
                .map(([expName, o]) => Object.entries(o))
                .reduce((a, fEntries) => [...a, ...fEntries], [])
                // only fields with primary == true
                .filter(([fName, o]) => o.primary)
                .map(([fName, o]) => fName)
                .reduce((s, fName) => s.add(fName), new Set())
                ;

                const defaultFields = Array.from(defaultFieldsSet);

                // tables have an arr of selected fields
                // and a float precision
                let content = {
                    itemName: `Table`,
                    fields: defaultFields,
                    precision: 10
                };

                scope.group.addReportItem(id, content);
            };

            // helper func for adding a plot
            scope.addNewPlot = (plot) => {
                let id = getNextItemId('plot');

                // plots have a given name (by analyzer)
                // and a plot type
                let content = {
                    itemName: `${plot.label}`,
                    name: plot.label,
                    type: plot.type,
                    merged: true,
                    savedPlotter: '',
                    savedConfig: ''
                };

                content.savedPlotter = PlotService.getPlotter(content).name;
                content.savedConfig = PlotService.getPlotterConfig(content).name;

                scope.group.addReportItem(id, content);
            };

            // helper func for adding a text block
            scope.addNewText = () => {
                let id = getNextItemId('text');
                // text blocks just have raw RST
                let content = {
                    itemName: 'Text',
                    text: ''
                };

                scope.group.addReportItem(id, content);
            };
        },
        template: `
<div class="btn-group" role="group">
    <button
        ng-disabled='group.experiments.length == 0'
        type="button"
        class="btn btn-default dropdown-toggle"
        data-toggle="dropdown"
        aria-haspopup="true"
        aria-expanded="false">
        Add Plot
        <span class="caret"></span>
    </button>
    <ul class='dropdown-menu' ng-repeat='(expName, plots) in plottables' ng-if='expName === group.experiments[0]'>
        <li ng-repeat='plot in plots'>
            <a ng-click='addNewPlot(plot)'>{{ plot.label }} <i>({{ plot.type }})</i></a>
        </li>
    </ul>
</div>
<button
    ng-disabled='group.experiments.length == 0'
    class='btn btn-default'
    ng-click='addNewTable()'>
    Add Table
</button>
<div class="btn-group" role="group">
    <button
        ng-disabled='group.experiments.length == 0'
        ng-click='addNewText()'
        type="button"
        class="btn btn-default"
        >
        Add Text Block
    </button>
</div>
`
    };
}]);
