/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
 */

/*
 * reportError
 * Desc:
 *  Watches the ErrorService's error list and processes an error if
 *  its not empty.
 *  'Processing': showing a modal to the user about the error.
 *
 */
angular.module('reportApp')
.directive("reportError", ['ErrorService', '$timeout', function(ErrorService, $timeout){
    return {
        scope: {
        },
        restrict: 'E',
        link: function(scope){
            scope.currError = {
                message: '',
                error: ''
            };

            const errors = [];

            const processError = () => {
                const modalOpen = ($("#errorReportModal").data('bs.modal') || {}).isShown;
                if(modalOpen || errors.length === 0){
                    return;
                }

                // save our error
                const e = errors.shift();
                scope.currError.message = e.message;
                scope.currError.error = e.error;
                scope.$apply();

                // pop up the modal
                $('#errorReportModal').modal();
            };

            $timeout(() => {
                $('#errorReportModal').on('hidden.bs.modal', () => {
                    // finished processing the last error
                    // process the next one
                    processError();
                });
            }, 0);

            scope.$on('user:error', (event, error) => {
                errors.push(error);
                processError();
            });
        },
        template: `
<bootstrap-modal dom-id='errorReportModal' button-cancel-text='Continue'>
    <b-title>
        Error
    </b-title>
    <b-content>
        <p>There was an error:</p>
        <p>{{ currError.message }}</p>
        <small>Details:</small>
        <pre>{{ currError.error }}</pre>
    </b-content>
</bootstrap-modal>
`
    };
}]);
