/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
 */

/*
 * experimentsTable
 * Desc:
 *  displays the report's experiments table
 *  - Experiment statuses
 *  - Name
 *  - Databases/Protocols
 *  - Analyzer
 *
 *  also lets the user remove experiments from the report.
 */
angular.module('reportApp')
.directive("experimentsTable", ['GroupsService', 'ExperimentsService', 'UrlService', 'ReportService', function(GroupsService, ExperimentsService, UrlService, ReportService){
    return {
        scope: {
        },
        link: function(scope){
            scope.ReportService = ReportService;
            scope.domId = `experiments-table`;
            scope.getAnalyzerFromExpName = ExperimentsService.getAnalyzerFromExpName;
            scope.getExpUrl = UrlService.getExperimentUrl;
            scope.getBlockUrl = UrlService.getBlockUrl;
            scope.getDatabaseUrl = UrlService.getDatabaseUrl;
            scope.getExperimentListPath = UrlService.getExperimentListPath;
            scope.isViewmode = UrlService.isViewmode;

            scope.expNames = ExperimentsService.experimentNames;
            scope.exps = ExperimentsService.experiments;

            scope.groups = GroupsService.groups;

            // collects an array of formatted databases & protocols of an experiment
            // format is "<database name>@<protocol name>"
            scope.getExpDatabases = (expName) => {
                let dbs = scope.exps[expName].declaration.datasets;
                return Array.from(new Set(Object.values(dbs).map(db => `${db.database}@${db.protocol}`)));
            };

            scope.deleteExpFromReport = (expName) => {
                ExperimentsService.deleteExperiment(expName);
            };
        },
        template: `
<div id='{{ domId }}' class='panel panel-default'>
    <div id='{{ domId }}-heading' class="panel-heading" role="tab">
        <h4 class="panel-title">
            <a
                class=''
                role="button"
                data-toggle="collapse"
                data-parent="#{{ domId }}-heading"
                href="#collapse-{{ domId }}"
                aria-expanded="true"
                aria-controls="collapse-{{ domId }}">
                Experiments List
            </a>
        </h4>
    </div>
    <div id="collapse-{{ domId }}"
        class="panel-collapse collapse in"
        role="tabpanel"
        aria-labelledby="{{ domId }}-heading">
            <table ng-if='expNames.length > 0' class="table table-striped table-hover">
                <thead>
                    <tr>
                        <th ng-if='!isViewmode()'></th>
                        <th ng-if='isViewmode() && groups.length == 1'>Alias</th>
                        <th>Experiment</th>
                        <th>Databases/Protocols</th>
                        <th>Analyzer</th>
                    </tr>
                </thead>
                <tbody>
                    <tr ng-repeat='expName in expNames'>
                        <td ng-if='!isViewmode()'>
                            <div class='btn-group action-buttons'>
                                <span
                                    ng-click='deleteExpFromReport(expName)'
                                    style='cursor: pointer;'
                                    class="btn-delete"
                                    data-toggle="tooltip"
                                    data-placement="top"
                                    title="Remove Experiment from Report">
                                    <i class="fa fa-times fa-lg"></i>
                                </span>
                            </div>
                        </td>
                        <td ng-if='isViewmode() && groups.length == 1'>
                            <span ng-if='groups[0].experiments.includes(expName)'>
                                {{ groups[0].aliases[expName] }}
                            </span>
                        </td>
                        <td><a href='{{ getExpUrl(expName) }}'>{{ expName }}</a></td>
                        <td>
                            <span ng-repeat='db in getExpDatabases(expName)'>
                                <a href='{{ getDatabaseUrl(db.split("@")[0]) }}'>{{ db }}</a>
                                &nbsp;
                            </span>
                        </td>
                        <td>{{ getAnalyzerFromExpName(expName) }}</td>
                    </tr>
                </tbody>
            </table>

            <div class='panel-body' ng-if='expNames.length == 0'>
                <i class="fa fa-warning fa-lg"></i>
                You have <strong>not added any experiments</strong> to this report yet.
                You may add experiments from throughout the platform
                (e.g. <a href="{{ getExperimentListPath() }}{{ ReportService.author }}/">your experiment list page</a>)
                to unlock editing features for this report.
            </div>
    </div>
</div>
`
    };
}]);
