/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
 */

/*
 * groupPanelContent
 * Desc:
 *  presents the group to the user in logical blocks:
 *  - a panel holds content
 *  - panel header contains an (editable) group name label
 *  - panel header also contains the buttons to add report items
 *  - two sub-panels are added: the experiments list, and the items list
 */
angular.module('reportApp').directive("groupPanelContent", ['GroupsService', 'UrlService', function(GroupsService, UrlService){
    return {
        scope: {
            group: '='
        },
        link: function(scope){
            scope.deleteGroup = GroupsService.deleteGroup;
            scope.isViewmode = UrlService.isViewmode;
        },
        template: `
<div panel-container class='dragGroup' domId='group.name'>
    <header>
        <a
            class=''
            role="button"
            data-toggle="collapse"
            data-parent="#{{group.name}}-heading"
            href="#collapse-{{group.name}}"
            aria-expanded="true"
            aria-controls="collapse-{{group.name}}">
        </a>
        <editable-label obj='group' field='_name'></editable-label>
        <div ng-if='!isViewmode()' class='btn-group action-buttons'>
            <span
                ng-click='deleteGroup(group.name)'
                class="btn btn-default btn-delete"
                data-toggle="tooltip" data-placement="top" title="Delete Group">
                <i class="fa fa-times fa-lg"></i>
            </span>
            <span drag-handle handle-helper-class='dragGroup'></span>
        </div>
        <div ng-if='!isViewmode()'
            group-add-items-menu
            class="btn-group" role="group" role='tab'
            group='group'>
        </div>
    </header>
    <content>
        <div group-panel-experiments group='group' class='panel panel-default'></div>
        <div
            style='margin-top: 5px;'
            ng-if='group.experiments.length > 0'
            group-panel-items
            group='group'
            >
        </div>
    </content>
</div>
`
    };
}]);

