/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
 */

/*
 * reportPublish
 * Desc:
 *  Displays a modal for publishing the current report.
 *  It also lets the user select which algorithms will be made
 *  Visible or Public.
 */
angular.module('reportApp')
.directive("reportPublish", ['reportFactory', 'ReportService', '$timeout', 'ErrorService', function(reportFactory, ReportService, $timeout, ErrorService){
    return {
        scope: {
        },
        restrict: 'E',
        link: function(scope, el){
            scope.algorithms = [];
            scope.radios = {};

            // sets up the data for the modal and opens it
            scope.buildPublishModal = () => {
                return reportFactory.publishReportAlgorithms(ReportService.author, ReportService.name, '')
                .then(res => {
                    // rm past data
                    scope.algorithms.splice(0, scope.algorithms.length);
                    Object.keys(scope.radios).forEach(k => delete scope.radios[k]);

                    // init new data
                    res.data.forEach(a => scope.algorithms.push(a));
                    scope.algorithms.forEach(a => { scope.radios[a] = ''; });
                })
                .catch(e => {
                    $('#publishReportModal').modal('hide');
                    // wait for the modal to close before logging error
                    setTimeout(() => ErrorService.logError(e, `Could not fetch the algorithms of the report, which is required to publish it.`), 500);
                })
                ;
            };

            $timeout(() => $('#publishReportModal').on('show.bs.modal', function(e){ return scope.buildPublishModal(); }), 0);

            // sends the request to publish the report, along with the algs chosen to be OS
            scope.publishReport = () => {
                const openSourceAlgs = Object.entries(scope.radios)
                .filter(([alg, val]) => val === 'openSource')
                .map(([alg, val]) => alg);

                const data = JSON.stringify({
                    visible_algorithms: openSourceAlgs
                });

                return ReportService.publishReport(openSourceAlgs)
                .then(() => {
                    window.location.reload();
                })
                .catch(e => {
                    ErrorService.logError(e, `Could not publish the report.`);
                })
                ;
            };
        },
        template: `
<bootstrap-modal dom-id='publishReportModal' button-submit-text='Publish' button-submit-func='publishReport'>
    <b-title>
        Publish Report
    </b-title>
    <b-content>
        <p>Publishing your report will make it accessible to anyone. Your report will not be editable anymore.</p>
        <div ng-if='algorithms.length > 0'>
            <p>The following algorithms will become public.
            They can either be <i>Visible</i> (no-one else can see inside the algorithm, but others can still use it) or <i>Open-Source</i> (the entire algorithm is visible as well as useable).
            Choose which algorithms will be Visible, and which will be Open-Source:</p>
            <form>
            <table class='table table-striped'>
                <thead>
                    <tr>
                        <td>Visible</td>
                        <td>Open-Source</td>
                        <td>Algorithm</td>
                    </tr>
                </thead>
                <tbody>
                    <tr ng-repeat='alg in algorithms'>
                        <td><input type='radio' name='{{ alg }}' ng-model='radios[alg]' value='visible'></td>
                        <td><input type='radio' name='{{ alg }}' ng-model='radios[alg]' value='openSource' checked></td>
                        <td>{{ alg }}</td>
                    </tr>
                </tbody>
            </table>
            </form>
        </div>
    </b-content>
</bootstrap-modal>
`
    };
}]);
