/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
 */

/*
 * reportSave
 * Desc:
 *  saves the current report
 */
angular.module('reportApp')
.directive("reportSave", ['GroupsService', 'ReportService', 'reportFactory', 'ErrorService', 'ExperimentsService', function(GroupsService, ReportService, reportFactory, ErrorService, ExperimentsService){
    return {
        restrict: 'A',
        link: function(scope, el){

            const saveReport = () => {
                // save the serialized group data...
                // the rest of the state is reconstructed from it and the URL
                let saveData = {
                    content: {
                        'groups': GroupsService.serializeGroups()
                    }
                };

                return reportFactory.removeExperiments(ExperimentsService.cachedDeletedExperiments)
                .then(() => reportFactory.updateReport(ReportService.author, ReportService.name, saveData, ''))
                .then(() => {
                    const lastEditedEl = document.querySelector('.lastEdited');
                    lastEditedEl.classList.remove('lastEditedAnimating');
                    void lastEditedEl.offsetWidth;
                    lastEditedEl.classList.add('lastEditedAnimating');
                    scope.$apply();
                })
                .catch(e => {
                    ErrorService.logError(e, `Could not save the report.`);
                });
            };

            el.bind('click', saveReport);
        },
    };
}]);
