/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
 */

/*
 * UrlService
 * Desc:
 *  Helper functionality to generate URLs for the reports app
 */
angular.module('reportApp').factory('UrlService', [function(){
    // const path segments
    const experimentSegment = 'experiments/';
    const blockSegment = 'algorithms/';
    const databaseSegment = 'databases/';
    const apiSegment = 'api/v1/';

    // the protocol, address, and port number
    let prefix = '';
    // the path to the current report, by username & report name
    let reportByName;
    // the path to the current report, by report number
    let reportByNumber;

    // extracts info from jQuery's ajaxSettings current URL
    const extractUsingCurrentUrl = () => {
        const url = $.ajaxSettings.url;
        const idxSplit = url.indexOf('reports');
        const path = url.slice(idxSplit).replace(/#\/?$/, '');

        prefix = url.slice(0, idxSplit);
        // find how many '/' are in path via splitting str on '/'
        if(path.split('/').length === 3){
            // report number
            reportByNumber = path;
        } else {
            // report user & name
            reportByName = path;
        }
    };

    const getPrefix = () => prefix;

    const getApiSegment = () => apiSegment;
    const getNameSegment = () => reportByName;
    const getNumberSegment = () => reportByNumber;

    const experimentPath = () => `${prefix}${experimentSegment}`;
    const blockPath = () => `${prefix}${blockSegment}`;
    const databasePath = () => `${prefix}${databaseSegment}`;
    const apiPath = () => `${prefix}${apiSegment}`;

    const getExperimentUrl = (experimentName) => `${experimentPath()}${experimentName}/`;
    const getBlockUrl = (blockName) => `${blockPath()}${blockName}/`;
    const getDatabaseUrl = (databaseName) => `${databasePath()}${databaseName}/`;
    const getApiUrl = (apiSubpath) => `${apiPath()}${apiSubpath}`;

    const getCompileRstUrl = () => `${getApiUrl(`${reportByName || reportByNumber}rst/`)}`;
    const getRemoveExperimentUrl = () => `${getApiUrl(`${reportByName}remove/`)}`;
    const getByNamePath = () => reportByName;
    const getByNumberPath = () => reportByNumber;
    const getExperimentListPath = () => experimentPath();

    const isViewmode = () => reportByNumber ? true : false;

    extractUsingCurrentUrl();

    return {
        getExperimentUrl,
        getBlockUrl,
        getDatabaseUrl,
        getCompileRstUrl,
        getRemoveExperimentUrl,
        getByNamePath,
        getByNumberPath,
        getApiSegment,
        getNameSegment,
        getNumberSegment,
        getPrefix,
        getExperimentListPath,
        isViewmode
    };
}]);
