/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
*/
/* Implementation of the new report dialog */

// Declaration of our namespaces
if (beat === undefined) var beat = {}
if (beat.reports === undefined) beat.reports = {}


/**
 * Uses bootstrap3 to display a modal window to create a report. The user is
 * presented with a "typeahead" input box so they can choose a name for their
 * report. The typeahead auto-completion works for the list of reports the user
 * current has on their portfoglio.
 *
 * Parameters:
 *
 *   url (String): A string with the URL to retrieve existing report
 *     names and to create reports
 *   username (String): The name of the user requesting the operation
 *
 **/
beat.reports.new_report = function (url, username) {

  //retrieve list of existing searches - if that succeeds, construct modal form
  function create() {

    return $.ajax({
      type: 'GET',
      url: url + '?fields=name',

    }).pipe(function(data) {

      var message = $(document.createElement('div'));
      message.append($(document.createElement('p')).text('By clicking Save, you will create a new report under the defined name. You can cancel the operation by clicking Cancel.'));
      var form_group = $(document.createElement('div'));
      form_group.addClass('form-group has-feedback');
      message.append(form_group);
      var input_group = $(document.createElement('div'));
      input_group.addClass('input-group');
      input_group.append($(document.createElement('span')).addClass('input-group-addon').text(username + ' /'));
      var input = $(document.createElement('input')).addClass('form-control');
      input.attr('type', 'text');
      input.attr('placeholder', 'Start typing for auto-completion...');
      if (name) input.val(name.split('/')[1]);
      input_group.append(input);
      glyph = $(document.createElement('span'));
      glyph.addClass('glyphicon glyphicon-ok form-control-feedback');
      glyph.attr('aria-hidden', 'true');
      input_group.append(glyph);
      form_group.append(input_group);

      var available_names = {};
      data.forEach(function(i){available_names[i.name.split('/')[1]]=i;});
      var available_name_keys = Object.keys(available_names);

      //visual feedback if new or overwriting...
      var help = $(document.createElement('span')).addClass('help');
      form_group.append(help);

      var update_help = function() {
        var currval = input.val();
        if (!currval.length || available_name_keys.indexOf(currval) >= 0) {
          help.text('Report name is invalid.');
          form_group.removeClass('has-success');
          form_group.addClass('has-error');
          glyph.removeClass('glyphicon-ok');
          glyph.addClass('glyphicon-remove');
        }
        else {
          help.text('');
          form_group.removeClass('has-error');
          form_group.addClass('has-success');
          glyph.removeClass('glyphicon-remove');
          glyph.addClass('glyphicon-ok');
        }
      }
      update_help(); //checks currently set value
      input.on('keyup', update_help);

      //using bootstrap3-typeahead
      input.typeahead({
        source: Object.keys(available_names),
        items: 5, //how many to show at a time
        minLength: 0, //show suggestions even if no text available
        autoSelect: false, //user may want to leave it where it is
        afterSelect: update_help, //updates after the user selected an item
        showHintOnFocus: true, //dropdown as soon as focus on
      });

      BootstrapDialog.show({
        title: '<i class="fa fa-floppy-o fa-lg"> Select a name',
        message: message,
        type: BootstrapDialog.TYPE_PRIMARY,
        buttons: [
        {
          label: 'Cancel',
          cssClass: 'btn-default',
          action: function(the_dialog) {
            the_dialog.close();
            return false;
          },
        },
        {
          label: 'Create',
          cssClass: 'btn-primary',
          action: function(the_dialog) {
            var val = input.val();
            if (!val) {
              BootstrapDialog.alert({
                title: '<i class="fa fa-warning"></i> Error',
                message: 'You must set a name to create your report',
                type: BootstrapDialog.TYPE_WARNING,
              });
              return false;
            }
            else if (available_name_keys.indexOf(val) >= 0) {
              BootstrapDialog.alert({
                title: '<i class="fa fa-warning"></i> Error',
                message: 'You must use a non-existing report name',
                type: BootstrapDialog.TYPE_WARNING,
              });
              return false;
            }
            the_dialog.close();

            if (val) {
              var d = $.ajax({
                type: 'POST',
                url: url,
                data: JSON.stringify({name: val}),
                contentType: "application/json; charset=utf-8",
                dataType: "json"
              });

              d.done(function(data, status) {
                the_dialog.close();
                location.reload();
                return true;
              });

              d.fail(process_error);

            } //val is set
          },
        },
        ],
      });
    });
  }

  return create().fail(process_error);

}
