// general tests for the reports app
describe('reports app', function(){
    // contain helpers for browser.wait
    const until = protractor.ExpectedConditions;
    // just use enter.perform() to send the enter key
    const enter = browser.actions().sendKeys(protractor.Key.ENTER);
    // since angular isnt configured correctly for protractor,
    // dont make protractor wait for angular
    browser.ignoreSynchronization = true;
    // just to make sure the window is maximized during these tests.
    // helps with button clicking & such
    browser.driver.manage().window().maximize();

    // login to the default user ('user') once before running all these tests
    beforeAll(function(){
        browser.get('http://localhost:8000/login/?next=/');
        //browser.findElement(by.partialLinkText('Sign-in')).click();
        browser.findElement(by.id('id_username')).sendKeys('user');
        browser.findElement(by.id('id_password')).sendKeys('user');
        browser.findElement(by.partialButtonText('Sign-in')).click();
        return browser.wait(function(){
            return browser.getCurrentUrl().then(function(url){
                const rxUserLoggedIn = /events\/user\//;
                return rxUserLoggedIn.test(url);
            });
        });
    });

    // if there's an error in the web browser's console,
    // fail the test and print the error
    afterEach(function() {
        let util = require('util');
        browser.manage().logs().get('browser').then(function(logs) {
            // 'failed to load resource' is a familiar log error
            // when running adblockers. Don't fail on those.
            let failingLogs = logs
            .filter(l => !(/net::ERR_FAILED/.test(l.message)));

            expect(failingLogs.length).toEqual(0);

            if (failingLogs.length > 0) {
                console.log(`logs: ${util.inspect(failingLogs)}`);
            }
        });
    });

    // /reports
    describe('home', function(){
        beforeEach(function(){
            browser.get('http://localhost:8000/reports/user');
        });

        it('should load', function(){
            expect(browser.getTitle()).toEqual("BEAT - user's Reports");
        });
    });

    // /reports/user
    describe('home for the test user', function(){

        // go to user's reports page before each test
        beforeAll(function(){
            browser.get('http://localhost:8000/reports/user/');
        });

        // before adding a report, there shouldn't be any
        it('should not have the "user/test" report', function(){
            let noReportsText = browser.findElement(by.css('.name > a'));
            expect(noReportsText.getText()).not.toBe('user/test');
        });

        // create a report
        it('should create a new report called "test"', function(){
            let newReportButton = browser.findElement(by.partialLinkText('New'));
            newReportButton.click();

            // wait for dialog box to pop up
            browser.sleep(1000);

            let reportNameInput = browser.findElement(by.css('.has-error input'));
            reportNameInput.sendKeys('test');

            let submitButton = element(by.buttonText('Create'));
            expect(submitButton.isPresent()).toBeTruthy();
            submitButton.click();

            // wait for page to refresh
            browser.sleep(1000);

            let newReportLink = browser.findElement(by.linkText('user/test'));
            newReportLink.click();
            expect(browser.getTitle()).toBe('BEAT - Report');
        });
    });

    // go to the experiments page and add up to 5 already-ran experiments
    describe('adding experiments to the "test" report', function(){
        // go to experiments page
        beforeAll(function(){
            browser.get('http://localhost:8000/experiments/user/');
        });

        it('should show the experiments list page', function(){
            expect(browser.getTitle()).toEqual("BEAT - user's Experiments");
        });

        it('should list successfully-ran experiments accessible by user', function(){
            expect(browser.isElementPresent(by.css('.Done'))).toBeTruthy();
        });

        it('should add up to the first 5 experiments to the "test" report', function(){
            let finishedExpTableRows = element.all(by.css('.Done'));
            let addButton = element(by.css('#add-to-report'));
            expect(addButton.getAttribute('disabled')).toBe('true');

            let fiveRows = finishedExpTableRows.filter((r, i) => i < 5);

            fiveRows
            .then(rs => Promise.all(rs.map(r => r.element(by.css('.report-checkbox')).element(by.css('input')).click())))
            .then(() => {
                browser.wait(until.elementToBeClickable(addButton), 5000, 'Button still isnt clickable!');
                return browser.executeScript('arguments[0].click();', addButton.getWebElement());
            })
            .then(() => browser.wait(until.presenceOf(element(by.css('.modal'))), 5000, 'Element taking too long to appear in the DOM'))
            .then(() => element(by.css('.chosen-single')).click())
            .then(() => element(by.css('.chosen-results')).element(by.css('.active-result')).click())
            .then(() => {
                let submitButton = element(by.buttonText('Add'));
                return submitButton.click();
            })
            .then(() => browser.wait(until.presenceOf(element(by.buttonText("View Report"))), 5000))
            .then(() => {
                let headerText = element(by.css('.report-results > h5'));
                expect(headerText.getText()).toContain('Successfully added');

                return browser.get('http://localhost:8000/reports/user/test/');
            })
            .then(() => {
                expect(element.all(by.css('#experiment-list-test > tbody > tr')).count()).toBeGreaterThan(0);
            })
            ;
        });
    });

    // create 2 groups, 'group1' & 'group2'
    describe('creating groups', function(){
        let newGroupInput = element(by.css('#createNewGroupInput'));

        it('should create the "group1" group using the enter key', function(){
            newGroupInput.sendKeys('group1')
            .then(() => enter.perform())
            .then(() => browser.wait(until.textToBePresentInElementValue(element(by.css('#createNewGroupInput')), ''), 1000))
            .then(() => expect(element.all(by.css('#groupsLayout > div')).count()).toBe(1))
            ;
        });

        it('should create the "group2" group using the "+" button', function(){
            newGroupInput.sendKeys('group2')
            .then(() => element(by.css('#space-for-report-items + div button')).click())
            .then(() => browser.wait(until.textToBePresentInElementValue(element(by.css('#createNewGroupInput')), ''), 1000))
            .then(() => expect(element.all(by.css('#groupsLayout > div')).count()).toBe(2))
            ;
        });
    });

    // make sure the initial report layouts page is correct
    describe('report page state with <6 experiments & two groups', function(){
        describe('header block', function(){
            const header = element(by.css('.col-sm-12 > p.bs-callout.bs-callout-danger'));

            it('has 4 labels', function() {
                expect(header.all(by.tagName('br')).count()).toBe(4);
            });

            it('shows the unique report id', function() {
                expect(header.element(by.css('.fa-arrow-circle-right + a')).getAttribute('href')).toMatch(/\/reports\/[0-9]+/);
            });

            it('shows the created date', function() {
                expect(header.element(by.css('.fa-calendar-o + strong')).getText()).toMatch(/.+ago/);
            });

            it('shows the "last edited" date', function() {
                expect(header.element(by.css('.fa-calendar-o + strong + br + .fa-calendar-o + strong')).getText()).toMatch(/.+ago/);
            });

            it('shows that the report is editable', function() {
                expect(header.element(by.css('.fa-warning + strong')).getText()).toBe('Editable');
            });
        });

        describe('documentation panel', function(){
            it('shows the empty warning', function() {
                expect(element(by.css('#description-display > div')).getAttribute('class')).toContain('alert-warning');
            });

            it('has an "Add" button', function() {
                expect(element(by.css('#btn-edit-doc > i')).getAttribute('class')).toContain('fa-edit');
            });
        });

        describe('experiments list panel', function(){
            describe('table', function(){
                const table = element(by.css('#experiment-list-test'));

                it('has 8 columns', function(){
                    const cols = table.all(by.css('thead th'));
                    expect(cols.count()).toBe(8);

                    expect(cols.get(0).getAttribute('class')).toBe('delete');
                    expect(cols.get(1).getAttribute('class')).toBe('attestation');
                    expect(cols.get(2).getAttribute('class')).toBe('privacy');
                    expect(cols.get(3).getAttribute('class')).toBe('status');
                    expect(cols.get(4).getAttribute('class')).toBe('date');
                    expect(cols.get(5).getText()).toBe('Name');
                    expect(cols.get(6).getAttribute('class')).toBe('datasets');
                    expect(cols.get(7).getAttribute('class')).toBe('analyzers');
                });

                it('has 5 or less rows', function() {
                    const rows = table.all(by.css('tbody > tr'));
                    expect(rows.count()).toBeLessThan(6);
                });

                it('has a "Remove Selected Experiments" button', function() {
                    const b = element(by.buttonText('Remove Selected Experiments'));
                    expect(b.getAttribute('disabled')).toBeDefined();
                });
            });
        });

        describe('report content block', function(){
            const groupsLayout = element(by.css('#groupsLayout'));

            describe('group1 container header', function(){
                const header = element(by.css('#group1-heading'));
                const children = element.all(by.css('#group1-heading > h4 > *'));

                it('has 5 children: a collapse link, name widget, a button group, and the add items menu', function(){
                    expect(children.count()).toBe(4);
                });

                describe('collapse link', function(){
                    it('toggles "#collapse-group1"', function(){
                        expect(children.get(0).getAttribute('href')).toContain('#collapse-group1');
                    });
                });

                describe('group name widget', function(){
                    const widgetEls = children.get(1).all(by.tagName('span'));
                    const label = widgetEls.get(0);
                    const button = widgetEls.get(1);

                    it('has value "group1"', function(){
                        expect(label.getText()).toBe('group1');
                    });

                    describe('edit button', function(){
                        it('has the pencil glyphicon', function(){
                            expect(button.getAttribute('class')).toBe('glyphicon glyphicon-pencil');
                        });

                        it('is clickable', function(){
                            // no direct way to check if clickable...
                            // so make sure that its not disabled & its displayed instead
                            expect(button.getAttribute('disabled')).toBeNull();
                            expect(button.isDisplayed()).toBeTruthy();
                        });
                    });

                });

                describe('button group', function(){
                    const grp = children.get(2);
                    const btnChildren = grp.all(by.className('btn'));

                    it('is a btn group', function(){
                        expect(grp.getAttribute('class')).toContain('btn-group');
                    });

                    it('is an action buttons', function(){
                        expect(grp.getAttribute('class')).toContain('action-buttons');
                    });

                    it('has btn children', function() {
                        const btnChildren = grp.all(by.className('btn'));
                        expect(btnChildren.count()).toBe(2);
                    });

                    it('has a button to delete a group with a red "X"', function(){
                        expect(btnChildren.get(0).getAttribute('title')).toBe('Delete Group');
                        expect(btnChildren.get(0).element(by.css('i')).getAttribute('class')).toContain('fa-times');
                    });

                    it('has a button to drag & sort the group with a 4-directional arrow', function(){
                        expect(btnChildren.get(1).getAttribute('title')).toBe('Drag to re-order group');
                        expect(btnChildren.get(1).element(by.css('i')).getAttribute('class')).toContain('fa-arrows');
                    });
                });

                describe('add items menu', function(){
                    const grp = children.get(3);
                    const buttons = grp.all(by.tagName('button'));

                    it('has 3 buttons', function(){
                        expect(buttons.count()).toBe(3);
                    });

                    it('is all disabled', function(){
                        expect(buttons.filter(b => b.getAttribute('disabled').then(d => d)).count()).toBe(3);
                    });

                    it('has a button to add plots', function(){
                        const el = element(by.buttonText('Add Plot'));
                        expect(el).toBeDefined();
                    });

                    it('has a button to add tables', function(){
                        const el = element(by.buttonText('Add Table'));
                        expect(el).toBeDefined();
                    });

                    it('has a button to add text blocks', function(){
                        const el = element(by.buttonText('Add Text Block'));
                        expect(el).toBeDefined();
                    });
                });
            });

            describe('group1 experiments panel', function(){
                const header = element(by.css('#group1-explist-heading'));
                const body = element(by.css('#collapse-group1-explist'));

                it('is visible', function(){
                    expect(header.isDisplayed()).toBeTruthy();
                    expect(body.isDisplayed()).toBeTruthy();
                });

                it('is empty', function(){
                    expect(element.all(by.css('#collapse-group1-explist > .panel-body > *')).count()).toBe(0);
                });

                describe('header button', function(){
                    const button = element(by.css('#group1_exp_add_dropdown'));

                    it('is non-disabled', function(){
                        expect(button.getAttribute('disabled')).toBeNull();
                    });

                    it('has the text "Add Experiment"', function(){
                        expect(button.getText()).toBe('Add Experiment');
                    });
                });
            });

            describe('group1 content panel', function(){
                it('doesnt exist', function(){
                    expect(element.all(by.css('#collapse-group1 > *')).count()).toBe(1);
                });
            });
        });
    });

    describe('group experiments panel management', function(){
        const g1ExpPanel = element(by.css('#collapse-group1 > .panel-body > .panel'));
        const addButton = element(by.css('#group1_exp_add_dropdown'));

        // add back the exp to group1
        afterAll(function(){
            const parent = addButton.element(by.xpath('..'));

            parent.getAttribute('class')
            .then(cls => cls.includes('open') || addButton.click())
            .then(() => {
                // make sure list opened correctly & add an exp
                const list = parent.element(by.css('ul'));
                const first = list.element(by.tagName('li'));
                browser.wait(until.elementToBeClickable(first), 5000, 'First add button isnt becoming clickable')

                return first.click();
            });
        });

        it('adds 1 experiment to group1', function(){
            // open add exp menu
            addButton.click()
            .then(() => {
                // make sure list opened correctly & add an exp
                const parent = addButton.element(by.xpath('..'));
                expect(parent.getAttribute('class')).toContain('open');
                const list = parent.element(by.css('ul'));
                const lis = list.all(by.css('li'));
                expect(lis.count()).toBeGreaterThan(0);

                return lis.get(0).element(by.css('a')).click();
            })
            .then(() => {
                // check state of group after adding exp
                const analyzer = element(by.css('#group1-explist-heading > h4 > i'));
                expect(analyzer.isDisplayed()).toBeTruthy();
                expect(analyzer.getText()).not.toBe('');

                const expsBody = element(by.css('#collapse-group1-explist > .panel-body'));
                expect(expsBody.getText()).not.toBe('');
            });
        });

        describe('group1 experiment table state with 1 experiment', function(){
            const expsTable = element(by.css('#collapse-group1-explist > .panel-body > table'));
            const headers = expsTable.all(by.css('thead > tr > th'));
            const row = expsTable.element(by.css('tbody > tr'));

            describe('column layout', function(){
                it('has 4 columns', function(){
                    expect(headers.count()).toBe(4);
                });

                it('has an empty col for rm exp buttons', function() {
                    expect(headers.get(0).getText()).toBe('');
                });

                it('has experiment names col', function() {
                    expect(headers.get(1).getText()).toBe('Experiment');
                });

                it('has dbs/protocols col', function() {
                    expect(headers.get(2).getText()).toBe('Databases/Protocols');
                });

                it('has aliases col', function() {
                    expect(headers.get(3).getText()).toBe('Alias');
                });
            });

            describe('row layout', function(){
                const cells = row.all(by.css('td'));

                it('has 4 entries', function() {
                    expect(cells.count()).toBe(4);
                });

                it('has a delete button in the first row', function() {
                    expect(cells.get(0).element(by.tagName('span')).getAttribute('class')).toContain('btn-delete');
                });

                it('has the experiment name and a link to the experiment in the second row', function() {
                    const a = cells.get(1).element(by.css('a'));
                    expect(a.getText()).not.toBe('');
                });

                it('has a list of formatted dbs & protocols in the third row', function() {
                    const els = cells.get(2).all(by.css('span'));
                    expect(els.count()).toBeGreaterThan(0);
                    expect(els.get(0).element(by.css('a')).getText()).toMatch(/\S+@\S+/);
                });

                it('has an editable alias input in the fourth row', function() {
                    const input = cells.get(3).element(by.css('input'));
                    const pExpName = cells.get(1).element(by.css('a')).getText();
                    pExpName
                    .then(expName => {
                        const lastSeg = expName.split('/').filter(s => s.length > 0).pop();
                        expect(input.getAttribute('value')).toBe(lastSeg);
                        expect(input.getAttribute('disabled')).toBeNull();
                    });
                });

            });
        });

        describe('removing 1 experiment', function(){
            beforeAll(function(){
                const rmExpButton = element(by.css('#collapse-group1-explist tbody .btn-delete'));
                return rmExpButton.click();
            });

            it('removes the exp table', function(){
                const explistPanel = element(by.css('#collapse-group1-explist > .panel-body'));
                expect(explistPanel.getText()).toBe('');
            });

            it('removes the analyzer tag', function() {
                expect(element.all(by.css('#group1-explist-heading > h4 > *')).count()).toBe(2);
            });

            it('lets the user add any experiment again', function() {
                const expRowsCount = element.all(by.css('#experiment-list-test tbody > tr')).count();

                Promise.all([expRowsCount, element(by.css('#group1_exp_add_dropdown')).click()])
                .then(([expCount]) =>
                    Promise.all([
                        expCount,
                        element('#group1_exp_add_dropdown').element(by.xpath('..')).element(by.css('ul')).all(by.css('li')).count()
                    ])
                )
                .then(([expCount, optsCount]) => {
                    expect(expCount).toBe(optsCount);
                });
            });
        });
    });

    // should have 1 exp in group
    describe('group report items panel management', function(){
        const addPlot = () => {
            const addButton = element(by.partialButtonText('Add Plot'));
            return addButton.click()
            .then(() => {
                const parent = addButton.element(by.xpath('..'));
                const firstPlotLi = parent.element(by.css('ul > li'));
                return firstPlotLi.click();
            })
            ;
        };

        const addTable = () => {
            const addButton = element(by.partialButtonText('Add Table'));
            return addButton.click();
        };

        const addTextBlock = () => {
            const addButton = element(by.partialButtonText('Add Text Block'));
            return addButton.click();
        };

        afterAll(function() {
            // add a plot, table, & text block
            return addPlot()
            .then(() => addTable())
            .then(() => addTextBlock())
            ;
        });

        describe('plot items', function(){
            describe('adding a plot', function() {
                beforeAll(function() {
                    // add plot here
                    return addPlot();
                });

                it('adds a plot', function() {
                    const plotContainer = element(by.css('#collapse-group1_plot_0'));
                    expect(plotContainer.isDisplayed()).toBeTruthy();
                });

                it('eventually renders the plot', function() {
                    const plotContainer = element(by.css('#collapse-group1_plot_0 > .panel-body > div'));

                    browser.wait(until.presenceOf(plotContainer.element(by.css('img'))),
                        5000, 'Plot render hasnt been inserted in Angular for 5s');

                    const img = plotContainer.element(by.css('img'));
                    expect(img.isDisplayed()).toBeTruthy();
                });
            });

            describe('removing a plot', function() {
                beforeAll(function() {
                    // rm plot here
                    const delButton = element(by.css('#group1_plot_0-heading .btn-delete'));

                    return delButton.click();
                });

                it('removes the plot and has no report items', function() {
                    const groupBodyChildren = element(by.model('group._reportItems')).all(by.css('*'));
                    expect(groupBodyChildren.count()).toBe(0);
                });
            });
        });

        describe('table items', function() {
            describe('adding a table', function() {
                beforeAll(function() {
                    // add table here
                    return addTable();
                });

                it('adds a table', function() {
                    const tableContainer = element(by.css('#collapse-group1_table_0 .panel-body'));
                    expect(tableContainer.isDisplayed()).toBeTruthy();
                });

                describe('layout', function() {
                    const panelHeaderButtonGroup = element.all(by.css('#group1_table_0-heading > h4 > .btn-group')).get(1);
                    const colsButton = panelHeaderButtonGroup.element(by.css('#group1_table_0_columnSelector'));
                    const precButton = panelHeaderButtonGroup.element(by.css('#group1_table_0-precision'));
                    const tcsvButton = panelHeaderButtonGroup.element(by.buttonText('Toggle CSV View'));

                    const table = element(by.css('#collapse-group1_table_0 table'));
                    const headers = table.all(by.css('thead > tr > th'));
                    const rows = table.all(by.css('tbody > tr'));

                    it('has at least one column for the name of the experiments', function() {
                        expect(headers.count()).toBeGreaterThan(0);
                        expect(headers.get(0).element(by.css('a')).getText()).toContain('Experiment');
                    });

                    it('has a default precision of 10', function() {
                        expect(precButton.getText()).toBe('Float Precision: 10');
                    });

                    it('has enabled button for choosing columns', function() {
                        expect(colsButton.getAttribute('disabled')).toBeNull();
                    });

                    it('has enabled button for choosing precision', function() {
                        expect(precButton.getAttribute('disabled')).toBeNull();
                    });

                    it('has enabled button for toggling the csv view', function() {
                        expect(tcsvButton.getAttribute('disabled')).toBeNull();
                    });

                    it('has 1 row with the name col matching the alias', function() {
                        const pName = rows.get(0).element(by.css('td')).getText();
                        const pCurrAlias = element(by.css('#collapse-group1-explist table > tbody input')).getAttribute('value');
                        Promise.all([pName, pCurrAlias])
                        .then(([name, currAlias]) => expect(name).toBe(currAlias));
                    });
                });
            });

            describe('removing a table', function() {
                beforeAll(function() {
                    // rm table here
                    const delButton = element(by.css('#group1_table_0-heading .btn-delete'));

                    return delButton.click();
                });

                it('removes the table and has no report items', function() {
                    const groupBodyChildren = element(by.model('group._reportItems')).all(by.css('*'));
                    expect(groupBodyChildren.count()).toBe(0);
                });
            });
        });

        describe('text block items', function() {
            describe('adding a text block', function() {
                beforeAll(function(){
                    // add text block here
                    return addTextBlock();
                });

                it('added a text block to the group', function() {
                    const textContainer = element(by.css('#collapse-group1_text_0 .panel-body'));
                    expect(textContainer.isDisplayed()).toBeTruthy();
                });

                it('is blank/empty', function() {
                    const emptyContainer = element(by.css('#collapse-group1_text_0 > .panel-body > .row > .col-sm-10 > div'));
                    expect(emptyContainer.getText()).toBe('');
                });

                it('shows the "edit" button', function() {
                    const editButton = element(by.css('#collapse-group1_text_0 > .panel-body > .row > .col-sm-2 > div > a'));
                    expect(editButton.getText()).toContain('Edit');
                });
            });

            describe('removing a text block', function() {
                beforeAll(function() {
                    // rm text here
                    const delButton = element(by.css('#group1_text_0-heading .btn-delete'));

                    return delButton.click();
                });

                it('removes the text and has no report items', function() {
                    const groupBodyChildren = element(by.model('group._reportItems')).all(by.css('*'));
                    expect(groupBodyChildren.count()).toBe(0);
                });
            });
        });
    });

    describe('collapse functionality', function() {
        // why are there all these 'sleep' calls before expanding the collapsed panel?
        // cuz, for some reason, theres a slight delay in it changing to the collapsed state in the DOM,
        // and it being available for being expanded
        describe('of groups', function() {
            const collapseButton = element(by.css('#group1-heading > h4 > a'));

            it('should collapse', function() {
                browser.executeScript('arguments[0].click();', collapseButton.getWebElement())
                .then(() => expect(collapseButton.getAttribute('class')).toBe('collapsed'))
                ;
            });

            it('should expand', function() {
                browser.sleep(500);
                browser.executeScript('arguments[0].click();', collapseButton.getWebElement())
                .then(() => expect(collapseButton.getAttribute('class')).toBe(''))
                ;
            });
        });

        describe('of experiments panel', function() {
            const collapseButton = element(by.css('#group1-explist-heading > h4 > a'));

            it('should collapse', function() {
                browser.executeScript('arguments[0].click();', collapseButton.getWebElement())
                .then(() => expect(collapseButton.getAttribute('class')).toBe('collapsed'))
                ;
            });

            it('should expand', function() {
                browser.sleep(500);
                browser.executeScript('arguments[0].click();', collapseButton.getWebElement())
                .then(() => expect(collapseButton.getAttribute('class')).toBe(''))
                ;
            });
        });

        describe('of plots', function() {
            const collapseButton = element(by.css('#group1_plot_0-heading > h4 > a'));

            it('should collapse', function() {
                browser.executeScript('arguments[0].click();', collapseButton.getWebElement())
                .then(() => expect(collapseButton.getAttribute('class')).toBe('collapsed'))
                ;
            });

            it('should expand', function() {
                browser.sleep(500);
                browser.executeScript('arguments[0].click();', collapseButton.getWebElement())
                .then(() => expect(collapseButton.getAttribute('class')).toBe(''))
                ;
            });
        });

        describe('of tables', function() {
            const collapseButton = element(by.css('#group1_table_0-heading > h4 > a'));

            it('should collapse', function() {
                browser.executeScript('arguments[0].click();', collapseButton.getWebElement())
                .then(() => expect(collapseButton.getAttribute('class')).toBe('collapsed'))
                ;
            });

            it('should expand', function() {
                browser.sleep(500);
                browser.executeScript('arguments[0].click();', collapseButton.getWebElement())
                .then(() => expect(collapseButton.getAttribute('class')).toBe(''))
                ;
            });
        });

        describe('of text blocks', function() {
            const collapseButton = element(by.css('#group1_text_0-heading > h4 > a'));

            it('should collapse', function() {
                browser.executeScript('arguments[0].click();', collapseButton.getWebElement())
                .then(() => expect(collapseButton.getAttribute('class')).toBe('collapsed'))
                ;
            });

            it('should expand', function() {
                browser.sleep(500);
                browser.executeScript('arguments[0].click();', collapseButton.getWebElement())
                .then(() => expect(collapseButton.getAttribute('class')).toBe(''))
                ;
            });
        });
    });

    describe('group experiment panel functionality', function() {
        const firstRowCells = element(by.css('#collapse-group1-explist tbody > tr')).all(by.css('td'));

        describe('alias field', function() {
            const alias = element(by.css('#collapse-group1-explist table > tbody input'));
            const strToType = 'asdf';

            it('lets you change the alias', function() {
                alias.sendKeys(strToType)
                .then(() => expect(alias.getAttribute('value')).toContain(strToType))
                ;
            });

            it('changed the experiment name in the first row in the table item', function(){
                const nameEl = element(by.css('#collapse-group1_table_0 td'));

                expect(nameEl.getText()).toContain(strToType);
            });
        });
    });

    describe('group report items functionality', function() {
        describe('plot', function() {
            const plotContainer = element(by.css('#group1_plot_0-render'));
        });

        describe('table', function() {
            const panelHeaderButtonGroup = element.all(by.css('#group1_table_0-heading > h4 > .btn-group')).get(1);
            const colsButton = panelHeaderButtonGroup.element(by.css('#group1_table_0_columnSelector'));
            const precButton = panelHeaderButtonGroup.element(by.css('#group1_table_0-precision'));
            const tcsvButton = panelHeaderButtonGroup.element(by.buttonText('Toggle CSV View'));

            const table = element(by.css('#collapse-group1_table_0 table'));
            const headers = table.all(by.css('thead > tr > th'));
            const rows = table.all(by.css('tbody > tr'));

            it('changes columns to include "total_execution_time"', function() {
                const b = element(by.partialButtonText('Choose Columns'));
                b.click()
                .then(() => {
                    const execTimeCol = element.all(by.css('#colSelectorId > .dropdown-menu > form > fieldset > div'))
                    .filter(e => e.element(by.tagName('input')).getAttribute('value').then(v => v == 'total_execution_time'))
                    .first()
                    ;

                    return execTimeCol.click();
                })
                .then(() => element(by.buttonText('Save Column Choices')).click())
                .then(() => {
                    const timeCols = element.all(by.css('#collapse-group1_table_0 thead a'))
                    .filter(a => a.getText().then(t => t.includes('total_execution_time')));

                    return expect(timeCols.count()).toBe(1);
                })
                ;
            });

            it('changes precision from 10 to 7', function() {
                const b = element(by.partialButtonText('Float Precision'));
                b.click()
                .then(() => element(by.linkText('7')).click())
                .then(() => expect(element(by.partialButtonText('Float Precision')).getText()).toContain('7'))
                .then(() => maps = element.all(by.css('#collapse-group1_table_0 tbody tr td'))
                    .map(td => td.getText()))
                .then(txts => txts
                    .filter(t => !isNaN(t) && t.includes('.'))
                    .forEach(t => expect(t.split('.')[1].length).toBeLessThan(8))
                )
                ;
            });
        });

        /*
         * the 'x' cancels tests from running
         * unfortunately there seems to be a selenium/protractor/webdriver bug
         * with looking into the codemirror editor (`ui-codemirror`)
         * none of the codemirror HTML elements are registering with
         * selenium/protractor as being visible
         */
        xdescribe('text block', function() {
            const getHtmlContainer = () => element(by.css('#collapse-group1_text_0 > .panel-body > .row > .col-sm-10 > div'));
            const getEditButton = () => element(by.css('#collapse-group1_text_0')).element(by.partialLinkText('Edit'));

            const text = '1. asdf';

            describe('edit mode', function() {
                it('turns on after clicking the edit button', function(){
                    getEditButton().click()
                    .then(() => {
                        browser.wait(until.presenceOf(element(by.css('.CodeMirror'))), 5000, 'Nope');
                        const editor = element.all(by.css('.CodeMirror'));
                        expect(editor.count()).toBeGreaterThan(0);
                    })
                    ;
                })

                it('has cancel button', function() {
                    expect(element(by.css('#collapse-group1_text_0')).element(by.partialLinkText('Cancel'))
                    .isDisplayed()).toBeTruthy();
                });

                it('has save button', function() {
                    expect(element(by.css('#collapse-group1_text_0')).element(by.partialLinkText('Save'))
                    .isDisplayed()).toBeTruthy();
                });

                it('lets you edit the RST', function() {
                    const textArea = element(by.css('.CodeMirror textarea'));
                    expect(textArea.isDisplayed()).toBeTruthy();

                    textArea.sendKeys(text)
                    .then(() => expect(textArea.getAttribute('value')).toBe(text))
                    ;
                });
            });

            describe('cancelling edits', function() {
                it('moves to view mode', function() {
                    element(by.css('#collapse-group1_text_0')).element(by.partialLinkText('Cancel'))
                    .click()
                    .then(() => expect(getEditButton().isDisplayed()).toBeTruthy())
                });

                it('didnt save the typed RST', function(){
                    expect(getHtmlContainer().getText()).toBe('');
                });
            });

            describe('submitting edits', function() {
                beforeAll(function(){
                    getEditButton().click()
                    .then(() => element(by.css('#collapse-group1_text_0 textarea')).sendKeys(text))
                    .then(() => element(by.css('#collapse-group1_text_0')).element(by.partialLinkText('Save')).click())
                    .then(() => browser.wait(until.presenceOf(element(by.css('#collapse-group1_text_0 ol.arabic.simple'))), 5000, 'HTML didnt render correctly'))
                    ;
                });

                it('renders the text correctly', function() {
                    expect(getHtmlContainer().isDisplayed()).toBeTruthy();

                    expect(getHtmlContainer().getText()).toBe(
                        `<ol class="arabic simple">
                        <li>asdf</li>
                        </ol>`
                    );
                });
            });
        });
    });

    // need to save report before doing these
    describe('links outside the report page', function(){
        beforeAll(function(){
            const saveButton = element(by.css('#save-button'));

            return browser.executeScript('arguments[0].click();', saveButton.getWebElement());
        });

        afterEach(function(){
            return browser.get('http://localhost:8000/reports/user/test/');
        });

        const firstRowCells = element(by.css('#collapse-group1-explist tbody > tr')).all(by.css('td'));
        const link1 = firstRowCells.get(1).element(by.css('a'));
        const link2 = firstRowCells.get(2).element(by.css('a'));

        describe('experiment link', function() {
            it('links to a valid experiment', function() {
                Promise.all([link1.getText(), link1.click()])
                .then(([expName]) => expect(browser.getTitle()).toBe(`BEAT - ${expName}`))
                ;
            });
        });

        describe('database~protocol link', function() {
            afterAll(function(){
                return browser.navigate().back();
            });

            it('links to a valid database~protocol', function() {
                Promise.all([link2.getText(), link2.click()])
                .then(([dbProtName]) => {
                    expect(browser.getTitle()).toBe(`BEAT - ${dbProtName.split('@')[0]}`)
                })
                ;
            });
        });
    });
});
