#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from .models import HourlyStatistics
from datetime import datetime

def updateStatistics(stats, date=None):
    """Update the hourly statistics

    :param beat.core.stats.Statistics statistics:
        Statistics collected during the processing done by an algorithm

    :param datetime.datetime date:
        Date and time the statistics were collected at. If None, use the
        current time.
    """

    date = date or datetime.now()

    # Retrieve the current obj entry (if it exists)
    try:
        obj = HourlyStatistics.objects.order_by('-date', '-hour')[0]
        if (obj.date != date.date()) or (obj.hour != date.hour):
            obj = None
    except:
        obj = None

    # Create an obj entry if necessary
    if obj is None:
        obj = HourlyStatistics()
        obj.date = date.date()
        obj.hour = date.hour

    # Modify the obj entry
    obj.cpu_time               += stats.cpu['user'] + stats.cpu['system']
    obj.max_memory             += stats.memory['rss']

    if 'volume' in stats.data:
      obj.data_read_size    += stats.data['volume'].get('read', 0)
      obj.data_written_size += stats.data['volume'].get('write', 0)

    if 'blocks' in stats.data:
      obj.data_read_nb_blocks    += stats.data['blocks'].get('read', 0)
      obj.data_written_nb_blocks += stats.data['blocks'].get('write', 0)

    if 'time' in stats.data:
      obj.data_read_time    += stats.data['time'].get('read', 0)
      obj.data_written_time += stats.data['time'].get('write', 0)

    obj.save()
