#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from rest_framework import permissions

from .models import Team

class IsOwner(permissions.BasePermission):
    """
    The logged in user should also be the owner
    """

    def has_object_permission(self, request, view, obj):
        return obj.owner == request.user

class HasPrivacyLevel(permissions.BasePermission):
    """
    Check whether the user has enough rights to view the team
    """

    def has_object_permission(self, request, view, obj):
        if obj.owner == request.user or obj.privacy_level == Team.PUBLIC:
            return True

        elif obj.privacy_level == Team.PRIVATE:
            return False

        elif obj.privacy_level == Team.MEMBERS and obj.members.filter(username=request.user.username):
            return True

        else:
            return False
