#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.web module of the BEAT platform.              #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

from django.db import models
from django.dispatch import receiver

from .models import Toolchain


#----------------------------------------------------------


# These two auto-delete files from filesystem when they are unneeded:
@receiver(models.signals.post_delete, sender=Toolchain)
def auto_delete_file_on_delete(sender, instance, **kwargs):
    """Deletes file from filesystem when ``Toolchain`` object is deleted.
    """
    if instance.declaration_file:
        instance.declaration_file.delete(save=False)

    if instance.description_file:
        instance.description_file.delete(save=False)


#----------------------------------------------------------


@receiver(models.signals.pre_save, sender=Toolchain)
def auto_delete_file_on_change(sender, instance, **kwargs):
    """Deletes file from filesystem when ``Toolchain`` object is changed."""

    if not instance.pk:
        return False

    try:
        old_file = Toolchain.objects.get(pk=instance.pk).declaration_file
    except Toolchain.DoesNotExist:
        return False

    if old_file != instance.declaration_file:
        old_file.delete(save=False)

    try:
        old_descr = Toolchain.objects.get(pk=instance.pk).description_file
    except Toolchain.DoesNotExist:
        return False

    if old_descr != instance.description_file:
        old_descr.delete(save=False)
