/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
*/
/* beat.toolchains.utils.js

   Toolchain-related utility functions
*/


// Declaration of our namespaces
if (beat === undefined)
    var beat = {}

if (beat.toolchains === undefined)
    beat.toolchains = {}

if (beat.toolchains.utils === undefined)
    beat.toolchains.utils = {}


//----------------------------------------------------------------------------------------
// Check that the given toolchain name is not already used by the user
//----------------------------------------------------------------------------------------
beat.toolchains.utils.check_toolchain_name = function(name, private, url_prefix, callback)
{
    var url;
    var toolchainPath = 'api/v1/algorithms/check_name/';

    if (url_prefix.endsWith('/'))
        url = url_prefix + toolchainPath;
    else
        url = url_prefix + '/' + toolchainPath;


    $.ajaxSetup({
        beforeSend: function(xhr, settings) {
            xhr.setRequestHeader('X-CSRFToken', $.cookie('csrftoken'));
        }
    });

    $.ajax({
        type: "POST",
        url: url,
        data: JSON.stringify({
            name: name,
            private: private,
        }),
        contentType: "application/json; charset=utf-8",
        dataType: "json",

        success: function(data) {
            callback(data.name, data.used, data.similar_to_private_version);
        },
    });
}



//----------------------------------------------------------------------------------------
// Load the given toolchain using the web API
//
// By default, only the toolchain declaration is retrieved from the API. To retrieve other
// fields, specify them via the 'additional_fields' parameter (as a comma-separated
// string).
//
// The callback is expected to have the following signature: callback(toolchain, data),
// where 'toolchain' is a beat.toolchains.models.Toolchain object and 'data' contains the
// additional fields.
//
// In case of error, 'toolchain' is null and 'data' is a string describing the error.
//----------------------------------------------------------------------------------------
beat.toolchains.utils.load = function(platform_url, toolchain_name, callback, additional_fields)
{
    // Fields processing
    var fields = 'declaration';

    if (additional_fields)
        fields += ',' + additional_fields;

    // Use the web API
    $.get(platform_url + '/api/v1/toolchains/' + toolchain_name + '/?fields=' + fields, function(data)
        {
            // Create the in-memory representation of the toolchain
            var toolchain = new beat.toolchains.models.Toolchain(data.declaration);

            // Remove the toolchain declaration from the web API response
            delete data.declaration;

            // Invoke the callback
            callback(toolchain, data);
        }
    ).fail(function(jqXHR) {
        if (jqXHR.status == 404)
            callback(null, 'Toolchain not found');
        else if (jqXHR.status == 403)
            callback(null, 'Unauthorized access');
        else
            callback(null, jqXHR.statusText);
    });
}
