/*
 * Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/
 * Contact: beat.support@idiap.ch
 *
 * This file is part of the beat.web module of the BEAT platform.
 *
 * Commercial License Usage
 * Licensees holding valid commercial BEAT licenses may use this file in
 * accordance with the terms contained in a written agreement between you
 * and Idiap. For further information contact tto@idiap.ch
 *
 * Alternatively, this file may be used under the terms of the GNU Affero
 * Public License version 3 as published by the Free Software and appearing
 * in the file LICENSE.AGPL included in the packaging of this file.
 * The BEAT platform is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU Affero Public License along
 * with the BEAT platform. If not, see http://www.gnu.org/licenses/.
*/
/* beat.ui.multipleselector.js

   Implementation of the multiple selector panel
*/


// Declaration of our namespaces
if (beat === undefined)
    var beat = {}

if (beat.ui === undefined)
    beat.ui = {}


/******************************* CLASS: MultipleSelector ********************************/

//----------------------------------------------------------------------------------------
// Constructor
//
// panel_id: ID of the DOM element to use
//----------------------------------------------------------------------------------------
beat.ui.MultipleSelector = function(panel_id)
{
    // Attributes
    this.id         = panel_id;
    this.element    = $('#' + panel_id)[0];
    this.background = null;
    this.entries    = null;

    // Callbacks
    this.onEntrySelected   = null;
    this.onEntryDeselected = null;
    this.onClose = null;
}


//----------------------------------------------------------------------------------------
// Displays the panel with the provided list of entries
//----------------------------------------------------------------------------------------
beat.ui.MultipleSelector.prototype.display = function(entries, left, top, width)
{
    var panel = this;

    this.entries = entries;

    if (width !== undefined)
        this.element.style.width = width + 'px';

    var ul = $(this.element).find('div.entries ul');
    ul.empty();
    ul = ul[0];

    for (var i = 0; i < entries.length; ++i)
    {
        var entry = this.entries[i];

        entry.row = document.createElement('li');
        entry.row.index = i;
        ul.appendChild(entry.row);

        var checkbox = document.createElement('input');
        checkbox.type = 'checkbox';
        checkbox.checked = entry.selected;
        entry.row.appendChild(checkbox);

        checkbox.entry = entry;

        if (entry.name.indexOf('.') >= 0)
        {
            var parts = entry.name.split('.');

            var span_prefix = document.createElement('span');
            span_prefix.className = 'prefix';
            span_prefix.textContent = parts[0] + '.';
            entry.row.appendChild(span_prefix);

            var span_name = document.createElement('span');
            span_name.className = 'name';
            span_name.textContent = parts[1];
            entry.row.appendChild(span_name);
        }
        else
        {
            var span_name = document.createElement('span');
            span_name.className = 'name';
            span_name.textContent = entry.name;
            entry.row.appendChild(span_name);
        }

        $(checkbox).change(function() {
            if (this.checked && panel.onEntrySelected)
                panel.onEntrySelected(this.entry.name, this.entry.data);
            else if (!this.checked && panel.onEntryDeselected)
                panel.onEntryDeselected(this.entry.name, this.entry.data);
            return false;
        });
    }

    var width         = $(this.element).width();
    var height        = $(this.element).height();
    var window_width  = $(window).width();
    var window_height = $(window).height();

    if (left === undefined)
    {
        left = (window_width - width) / 2;
    }
    else
    {
        left -= $(window).scrollLeft();

        if (left + width >= window_width)
            left = window_width - width - 4;
    }

    if (top === undefined)
    {
        top = (window_height - height) / 2;
    }
    else
    {
        top -= $(window).scrollTop();

        if (top + height >= window_height)
            top = window_height - height - 4;
    }

    this.element.style.top  = '' + top + 'px';
    this.element.style.left = '' + left + 'px';

    this.background = document.createElement('div');
    this.background.className = 'ui-widget-overlay ui-front';

    $(this.background).insertBefore(this.element);


    // Behavior
    $(this.background).click(function() {
        $(panel.element).hide();
        $(panel.background).remove();
        panel.background = null;
        if(panel.onClose != null)
        {
            panel.onClose();
        }
        return false;
    });

    $(this.element).show();
}
